/*** 
 * \file gre/3dcdgeom.h
 * \brief CAD_GEOMETRIC3D algorithms
 *
 * \if NODOC
 * $Id: 3dcdgeom.h_v 1.1 2007/06/14 22:59:44 vdronov Exp $
 *
 * $Log: 3dcdgeom.h_v $
 * Revision 1.1  2007/06/14 22:59:44  vdronov
 * Initial revision
 *
 * Revision 1.1  2007/06/06 17:39:20  vdronov
 * Initial revision
 *
 *
 * \endif
**/ 

#ifndef	INC_GRE_3DCDGEOM_H
#define	INC_GRE_3DCDGEOM_H

#ifndef	INC_GRE_3DGEOMET_H
#include <gre/3dgeomet.h>
#endif

#ifndef	INC_RVC_CAD_H
#include <rvc/cad.h>
#endif

#ifndef	INC_GRE_LRCAD_H
#include <gre/lrcad.h>
#endif

namespace GRE {

class CAD_GEOMETRIC3D : public GEOMETRIC3D {
	public:

		CAD_GEOMETRIC3D (			 							  
			);

		virtual ~CAD_GEOMETRIC3D (								//! Destructor
			);

		RVC::CAD* GetCAD () {return &m_CAD; };
		GRE::LAYER_CAD* GetCADLayer () { return m_Layer; };
		double ComputeScale (const INT32 id);
		void Set (
			GRE::LAYERDC* layerdc,
			GRE::LAYER_CAD::DISPPARM* dispparm
			);
		void Clear();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		RVC::CAD m_CAD;
		GRE::LAYER_CAD *m_Layer;
		DRECT3D ALIGN16(m_Extents);
		SPATREF::COORDREFSYS m_CoordRefSys;
		TRANS2D_MAPGEN m_Transformation;
		GRE::LAYERDC* m_LayerDC;
		GRE::LAYER_CAD::DISPPARM* m_DispParm;

		CAD_GEOMETRIC3D (											//! Copy constructor (not implemented)
			const CAD_GEOMETRIC3D& rhs
			);

		CAD_GEOMETRIC3D& operator= (								//! Assignment operator (not implemented)
			const CAD_GEOMETRIC3D& rhs
			);

		ERRVALUE UpdateExtents (
			TRANS2D_MAPGEN& transformation, 
			DRECT3D& extents
            );

		bool IsTerrainNeeded () { return true; }
		bool IsPointsFieldNeeded () { return true; }

		GEOMETRIC3D* v_CreateInstance (
			) { return new CAD_GEOMETRIC3D(); };

		ERRVALUE v_Build (
			const GRE::LAYER *layer
			);

		void v_Free (
			);

		const DRECT3D& v_GetExtents (
			) { return m_Extents; };

		void v_SetCoordRefSys (
			const SPATREF::COORDREFSYS& CoordRefSys
			);

		const SPATREF::COORDREFSYS v_GetCoordRefSys (
			) { return m_CoordRefSys; };

		const TRANS2D_MAPGEN& v_GetTransformationInternalToModel (
			) { return m_Transformation; };

		void OnDrawBegin (	GRE::GEOMETRIC3D_BUILDER* builder, const bool wireframe = false);

		void OnDrawEnd ();


		#endif // GENERATING_DOXYGEN_OUTPUT

	};

}		// End of GRE namespace

#endif



