/**
 * \file gre/dlgmaskedit.h
 *
 * \if NODOC
 * $Id: dlgmaskedit.h_v 1.25 2013/04/02 16:49:18 bcolombe Exp $
 *
 * $Log: dlgmaskedit.h_v $
 * Revision 1.25  2013/04/02 16:49:18  bcolombe
 * Don't override Suspend and Resume
 *
 * Revision 1.24  2012/12/04 22:20:09  mju
 * Make some methods non-static so can use with drawgen delegates.
 *
 * Revision 1.23  2012/08/13 19:38:49  bcolombe
 * Added fillrect
 *
 * Revision 1.22  2011/12/05 17:44:49  bcolombe
 * Add GetLocatorView
 *
 * Revision 1.21  2011/08/26 13:23:37  mju
 * Use formradiobuttons hdr.
 *
 * Revision 1.20  2010/01/14 17:26:56  bcolombe
 * Use single toolbar
 *
 * Revision 1.19  2009/06/23 20:01:10  mju
 * Use getObjItem.
 *
 * Revision 1.18  2008/10/16 13:13:16  mju
 * Remove mxtools hdr.
 *
 * Revision 1.17  2008/10/07 16:34:52  bcolombe
 * Added RCB_Mark
 *
 * Revision 1.16  2008/09/08 20:05:12  bcolombe
 * Add pane to tooldata
 *
 * Revision 1.15  2008/07/18 21:45:33  bcolombe
 * Added ability to expand/shrink marked area with tool
 *
 * Revision 1.14  2008/07/17 21:38:16  bcolombe
 * Removed ObjectViewSetRast
 *
 * Revision 1.13  2008/06/24 16:51:45  bcolombe
 * Use new view and tools
 *
 * Revision 1.12  2007/11/12 20:28:38  bcolombe
 * Use GRE::TOOL
 *
 * Revision 1.11  2007/11/12 15:03:15  mju
 * Remove unneeded legacy hdrs.
 *
 * Revision 1.10  2007/10/02 15:23:42  scowan
 * Changed createDialog to createMaskDialog because under mfc, it conflicts
 * with windows macro.
 *
 * Revision 1.8  2007/08/01 22:08:52  bcolombe
 * Moved clear mask button to mask edit toolbar
 *
 * Revision 1.7  2007/07/30 20:56:23  bcolombe
 * Added onchange callback for tools
 *
 * Revision 1.6  2007/07/20 15:40:35  bcolombe
 * Added read/write for mask that takes void* and converts to INT8
 * Added own fillseeded function
 *
 * Revision 1.5  2007/07/18 22:39:12  bcolombe
 * Removed some unused extents code
 *
 * Revision 1.4  2007/07/17 21:21:19  bcolombe
 * Change get/setfillval to work with multi-band rasters
 *
 * Revision 1.3  2007/07/16 17:28:08  bcolombe
 * Added 'tolerance' settings to floodfill
 *
 * Revision 1.2  2007/07/02 19:00:34  bcolombe
 * Added private function for inverting mark layer
 *
 * Revision 1.1  2007/06/18 17:30:38  bcolombe
 * Initial revision
 * \endif
****************************************************************************/

#ifndef	INC_GRE_DLGMASKEDIT_H
#define	INC_GRE_DLGMASKEDIT_H

#ifndef INC_RVC_RASTER_H
	#include <rvc/raster.h>
#endif
#ifndef INC_RVC_UIFORMOBJECTSINGLE_H
	#include <rvc/uiformobjectsingle.h>
#endif

#ifndef INC_MI32_MG2SIMPL_H
	#include <mi32/mg2simpl.h>
#endif
#ifndef INC_MI32_DRAWGEN_H
	#include <mi32/drawgen.h>
#endif
#ifndef INC_MI32_REGION2D_H
#include <mi32/region2d.h>
#endif

#ifndef INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif
#ifndef INC_MGUI_COLORSEL_H
#include <mgui/colorsel.h>
#endif
#ifndef INC_MGUI_FORMRADIOBUTTONS_H
#include <mgui/formradiobuttons.h>
#endif
#ifndef INC_MGUI_TOOLBAR_H
#include <mgui/toolbar.h>
#endif
#ifndef INC_MGUI_SLIDER_H
#include <mgui/slider.h>
#endif
#ifndef INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

#ifndef INC_GRE_UIFORMVIEW_H
#include <gre/uiformview.h>
#endif
#ifndef INC_GRE_UIVIEWCANVAS_H
#include <gre/uiviewcanvas.h>
#endif
#ifndef INC_GRE_UIVIEWTOOL_H
#include <gre/uiviewtool.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
namespace GRE {
	class LAYER_RASTER;
	class GROUP;
	class VIEWABLE;
	class TOOL;
	}
typedef void *MXTHANDLE;
struct MxtAnyCallbackStruct;
struct MrgnManagerCallbackStruct;
#endif // GENERATING_DOXYGEN_OUTPUT


namespace GRE {

//! Class to create a dialog for editing a raster mask
class DLG_MASKEDIT : public MGUI::DLGSHELL {
	public:

		DLG_MASKEDIT (
			MDLGPARENT DlgParent = NULL,			//!< DlgParent of process, null if process is standalone
			GRE::LAYER_RASTER *RasterLayer = NULL	//!< Raster layer to use, null if needs to be created
			);

		~DLG_MASKEDIT (
			);

		//! Create the dialog
		void Create ();

		//! Edit a raster's mask from a layer
		ERRVALUE SelectRaster (
			GRE::LAYER_RASTER *SrcRastLayer = NULL //!< Layer to edit mask for
			);

	private:	
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DLG_MASKEDIT THISCLASS;

		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnCloseRequest ();
		virtual void v_OnCancel ();
		virtual void v_OnOK();

		//! Class for creating/editing a temporary binary raster used to create the mask
		class MASK {
			public:

				MASK ();
				~MASK ();

				//! Close the raster
				void Close (
					);

				//! Create the mask from a binary raster
				ERRVALUE CopyToTempFrom (
					const RVC::OBJITEM SrcObjItem	//!< Objitem of binary raster
					);

				//! Apply operation to entire mask
				ERRVALUE FillAll (
					UINT8 operation	//!< Exclue or include all of mask
					);

				//! Fill region of mask
				ERRVALUE FillRegion (
					const REGION2D& region,	//!< Region to fill
					bool value				//!< Value to fill region with
					);

				//! Fill rect of mask
				ERRVALUE FillRect (
					INT32 x1,
					INT32 y1,
					INT32 x2,
					INT32 y2,
					bool value				//!< Value to fill region with
					);

				//! Get the buffer containing the currently read line
				UINT8* GetBuffer (
					) { return (m_buffer); }

				//! Get the mask's MAKEPARMS
				const RVC::RASTER::MAKEPARMS& GetMakeParms (
					) { return (m_ObjRaster.GetMakeParms()); }

				//! Get mask's object item
				ERRVALUE GetObjItem (
					RVC::OBJITEM& ObjItem
					) { return (m_ObjRaster.GetObjItem(ObjItem)); }

				//! Create an empty temporary raster
				ERRVALUE MakeTemp (
					INT32 NumColumns,	//!< Number of columns in mask
					INT32 NumLines		//!< Number of lines in mask
					);

				//! Read a line from the mask
				ERRVALUE Read (
					INT32 line,			//!< Line to read
					INT32 initcol,		//!< Starting column
					UINT8 *buffer,		//!< Data read
					INT32 numcols		//!< Number of columns to read
					) { return (m_ObjRaster.Read(line, initcol, buffer, numcols)); }

				//! Read a line from the mask
				ERRVALUE Read (
					INT32 line,			//!< Line to read
					INT32 initcol,		//!< Starting column
					void *buffer,		//!< Data read
					INT32 numcols		//!< Number of columns to read
					) { return (m_ObjRaster.Read(line, initcol, buffer, numcols, RVC::IMAGE::CONVMODE_INT8)); }

				//! Write a line to the mask
				ERRVALUE Write (
					INT32 line,			//!< Line to write
					INT32 initcol,		//!< Starting column
					UINT8 *buffer,		//!< Data to write
					INT32 numcols		//!< Number of columns to write
					) { return (m_ObjRaster.Write(line, initcol, buffer, numcols)); }

				//! Write a line to the mask
				ERRVALUE Write (
					INT32 line,			//!< Line to write
					INT32 initcol,		//!< Starting column
					void *buffer,		//!< Data to write
					INT32 numcols		//!< Number of columns to write
					) { return (m_ObjRaster.Write(line, initcol, buffer, numcols, RVC::IMAGE::CONVMODE_INT8)); }

				//! Copy the mask
				ERRVALUE CopyRasterTo (
					RVC::RASTER& TgtRaster	//!< Target raster to copy to
					) { return (m_ObjRaster.CopyRasterTo(TgtRaster)); }

				ERRVALUE CopyToNullMask (
					RVC::RASTER& TgtRaster	//!< Target raster to copy to
					);

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				RVC::RASTER m_ObjRaster;			//!< Raster used for mask
				SIMPLE_ARRAY<UINT8> m_buffer;		//!< Buffer to read/write from

				//! Fill a span
				static ERRVALUE OnFillSpan (		
					MGD::CONTEXT_SIMPLE *dc,
					INT32 initcol,
					INT32 lastcol,
					INT32 lin
					);

				//! Fill a span
				ERRVALUE FillSpan (					
					INT32 initcol,
					INT32 lastcol,
					INT32 lin
					);

				//! Set cache size and resize buffer
				ERRVALUE Setup ();
				#endif
			};

		class MASKEDITTOOLINFO{
			public:
				MASKEDITTOOLINFO () {m_pane = new MGUI::LAYOUT_PANE;}
				~MASKEDITTOOLINFO () {};

				MXTHANDLE GetHandle() {return m_handle;}
				void SetHandle(MXTHANDLE handle) {m_handle = handle;}

				Widget GetMainForm() {return m_mainform;}
				void SetMainForm(Widget mainform) {m_mainform = mainform;}

				Widget GetManualForm() {return m_manualform;}
				void SetManualForm(Widget manualform) {m_manualform = manualform;}

				MGUI::LAYOUT_PANE* GetPane() {return m_pane;}
				void SetPane(MGUI::LAYOUT_PANE* pane) {m_pane = pane;}

			private:
				MGUI::LAYOUT_PANE *m_pane;
				MXTHANDLE m_handle;
				Widget m_mainform;
				Widget m_manualform;
			};

		class MASKEDITTOOLDATA : public GRE::UI::VIEWTOOL {
			public:

				MASKEDITTOOLDATA (
					GRE::UI::VIEW_CANVAS *view
					);

				virtual ~MASKEDITTOOLDATA ();

				Widget GetMainForm() {return m_mainform;}
				void SetMainForm(Widget mainform) {m_mainform = mainform;}

				Widget GetManualForm() {return m_manualform;}
				void SetManualForm(Widget manualform) {m_manualform = manualform;}

				UINT8 GetToolType() {return m_tooltype;}
				void SetToolType(UINT8 tooltype) {m_tooltype = tooltype;}

				MASKEDITTOOLINFO* GetToolInfo(UINT8 tooltype) {return &m_toolinfo[tooltype];}

				DLG_MASKEDIT* GetPdp() {return m_pdp;}
				void SetPdp(DLG_MASKEDIT *pdp) {m_pdp = pdp;}

				INT32 m_BufferPixels;
				REGION2D m_LastRegion;
				bool m_ToolIsCleared;

			private:
				//:> MEMBERS

				MASKEDITTOOLINFO m_toolinfo[8];
				UINT8 m_tooltype;
				Widget m_mainform;							// Main Form widget
				Widget m_manualform;						// Form widget for manual edit tool form
				DLG_MASKEDIT *m_pdp;

				//:> VIRTUAL METHODS

				virtual bool v_Activate ();
				virtual void v_Deactivate ();
				virtual bool v_Initialize ();

				//:> METHODS

				void UpdateTrans ();

				};

		//! Class containing group, view and layers
		class VIEWDATA : public MGUI::DLGSHELL {
			public:

				VIEWDATA ();
				~VIEWDATA ();

				GRE::VIEWABLE* GetViewable() {return m_viewable;}
				void SetViewable(GRE::VIEWABLE *viewable) {m_viewable = viewable;}

				GRE::UI::VIEW_CANVAS* GetLocatorView() {return m_formview->GetViewLocator();}
				GRE::UI::VIEW_CANVAS* GetView() {return m_view;}
				void SetView(GRE::UI::VIEW_CANVAS* view) {m_view = view;}

				GRE::GROUP* GetGroup() {return m_group;}
				void SetGroup(GRE::GROUP* group) {m_group = group;}

				MASKEDITTOOLDATA* tooleditmask;

				GRE::LAYER_RASTER* GetRasterLayer() {return m_rastlayer;}
				void SetRasterLayer(GRE::LAYER_RASTER *layer) {m_rastlayer = layer;}

				GRE::LAYER* GetMaskLayer() {return m_masklayer;}
				void SetMaskLayer(GRE::LAYER *layer) {m_masklayer = layer;}

				GRE::LAYER* GetMarkLayer() {return m_marklayer;}
				void SetMarkLayer(GRE::LAYER *layer) {m_marklayer = layer;}

				COLOR GetMaskIncludeColor() {return m_MaskIncludeColor;}
				void SetMaskIncludeColor(COLOR color) {m_MaskIncludeColor = color;}

				COLOR GetMaskExcludeColor() {return m_MaskExcludeColor;}
				void SetMaskExcludeColor(COLOR color) {m_MaskExcludeColor = color;}

				COLOR GetMarkColor() {return m_MaskMarkColor;}
				void SetMarkColor(COLOR color) {m_MaskMarkColor = color;}

			private:
				virtual ERRVALUE v_CreateContent();

				GRE::VIEWABLE *m_viewable;
				GRE::UI::VIEW_CANVAS *m_view;
				GRE::UI::FORM_VIEW *m_formview;
				GRE::GROUP *m_group;
				GRE::LAYER_RASTER *m_rastlayer;
				GRE::LAYER *m_masklayer;
				GRE::LAYER *m_marklayer;
				COLOR m_MaskIncludeColor;
				COLOR m_MaskExcludeColor;
				COLOR m_MaskMarkColor;
			};

		//! Class for selecting a range of cell values
		class THRESHDATA {
			public:

				THRESHDATA ();
				~THRESHDATA () {};

				double GetUserRangeBegin() {return m_UserRangeBegin;}
				void SetUserRangeBegin(double range) {m_UserRangeBegin = range;}

				double GetUserRangeEnd() {return m_UserRangeEnd;}
				void SetUserRangeEnd(double range) {m_UserRangeEnd = range;}

				DOUBLE_RANGE GetRasterRange() {return m_RasterRange;}
				void SetRasterRange(DOUBLE_RANGE range) {m_RasterRange = range;}

				bool IsInside() {return m_IsInside;}
				void SetInside(bool inside) {m_IsInside = inside;}

				bool IsInteger() {return m_IsInteger;}
				void SetInteger(bool integer) {m_IsInteger = integer;}

			private:
				double m_UserRangeBegin;			//!< User defined beginning of range
				double m_UserRangeEnd;			//!< User defined end of range
				DOUBLE_RANGE m_RasterRange;		//!< rastmin, rastmax
				bool m_IsInside;					//!< Values should be selected outside or inside the defined range
				bool m_IsInteger;
		   };

		VIEWDATA m_viewdata;
		THRESHDATA *m_threshdata;
		
		enum SAVEMODE {
			SAVEMODE_Mask =			0,		//!< Save as binary raster mask
			SAVEMODE_NullMask =		1		//!< Save as null mask under raster
			};

		INT32 GetRasterNumLins() {return m_RasterNumLins;}
		INT32 GetRasterNumCols() {return m_RasterNumCols;}
		void SetRasterNumCols(INT32 cols) {m_RasterNumCols = cols;}
		void SetRasterNumLins(INT32 lins) {m_RasterNumLins = lins;}

		//! Get the cell value to be searched for when flood filling
		double GetFillVal(INT32 i) {return m_FillVal[i];}
		//! Get the cell value range for filling
		double GetFillRange() {return m_FillRange;}
		//! Set the cell value to be searched for when flood filling
		void SetFillVal(INT32 i, double FillVal) {m_FillVal[i] = FillVal;}
		//! Set the cell value range for filling
		void SetFillRange(double FillRange) {m_FillRange = FillRange;}

		//! Set whether mask needs to be saved on exit
		void SetUserSaved(bool Saved) {m_UserSaved = Saved;}
		void SetSaveMode(SAVEMODE mode) {m_SaveMode = mode;}
		SAVEMODE GetSaveMode() {return m_SaveMode;}	
		bool IsUserSaved() {return m_UserSaved;}

		RVC::OBJITEM GetMaskObjItem() {return m_MaskObjItem;}
		void SetMaskObjItem(const RVC::OBJITEM& objitem) {m_MaskObjItem = objitem;}
		void ClearMaskObjItem() {m_MaskObjItem.Clear();}

		ERRVALUE ObjectViewOpen ();
		void ObjectViewSetupMask ();
		int SetupMask (bool UseSavedMask = false);

		void SetDlgParent(MDLGPARENT DlgParent) {m_DlgParent = DlgParent;}
		MDLGPARENT GetDlgParent() {return m_DlgParent;}

		ERRVALUE SelectMask (
			bool forceselect					// Force selection even if already have one set
			);

		void SetMaskColors ();
		void SetMarkColors ();

		void ReadDefaults ();
		void SaveDefaults ();
		void RunProcess ();

		//! Find georeference object and read transformation
		TRANS2D_MAPGEN GetFirstGeorefTrans() {return m_FirstGeorefTrans;}
		void SetFirstGeorefTrans(TRANS2D_MAPGEN trans) {m_FirstGeorefTrans = trans;}
		void GetTransLayerDispAffine (MAT3X3 OtoD, MAT3X3 DtoO);
		int ObjectViewSetupMaskGeoref();

		//Tool callbacks
		static void TCB_MaskEditToolActivate (MxtAnyCallbackStruct *cbstr, void *vtdp);
		static void TCB_MaskEditToolChange (MxtAnyCallbackStruct *cbstr, void *vtdp);
		static void MaskEditToolSetTrans(MASKEDITTOOLDATA *tdp);
		static void RCB_SetTransParm (MrgnManagerCallbackStruct *cb, void *vtdp);
		static void RCB_Mark (MrgnManagerCallbackStruct *cb, void *vtdp);

		void MaskEditToolOp (MASKEDITTOOLDATA *tdp, int op);
		//Test cell value function for flood filling
		int TestInside (INT32 col, INT32 lin);

		//Drawing function for flood filling
		ERRVALUE DrawSpan (INT32 start, INT32 end, INT32 lin);

		MDLGPARENT m_DlgParent;
		GRE::LAYER_RASTER *m_RasterLayer;
		GRE::UI::VIEWTOOL *m_PrevTool;

		RVC::OBJITEM m_MaskObjItem;
		SAVEMODE m_SaveMode;
		bool m_UserSaved;
		bool m_IsDestroyLayer;
		DOUBLE_ARRAY<double> m_FillVal;
		double m_FillRange;
		TRANS2D_MAPGEN m_FirstGeorefTrans;				// Georef transformation for first selected raster
		INT32 m_RasterNumLins;							// Lines in first selected raster
		INT32 m_RasterNumCols;							// Columns in first selected raster

		RVC::RASTER m_ObjSrcRaster;
		MASK m_SrcMask;
		MASK m_MarkMask;

		//MGUI:
		ERRVALUE CreateMaskTools (MGUI::LAYOUT_PANE_BASE& ParentPane);
		ERRVALUE CreateMaskDialog (MGUI::LAYOUT_PANE& ParentPane);

		void OnApplyRange ();
		void OnModifyPressed ();
		void OnSelectRaster ();
		void OnMaskSpecify ();
		void OnChangeHighlightColor ();
		void OnSliderChanged ();
		void OnShowManual ();
		void OnExit ();
		void OnToolBar(MGUI::ID cmdid, bool state);

		void MaskEditToolSetup (UINT8 tooltype);

		RVC::UI::FORM_OBJECT_SINGLE m_SrcRasterSelect;

		MGUI::CTRL_GROUPBOX m_colorGroupBox;
		MGUI::CTRL_PUSHBUTTON m_MaskSelect;
		MGUI::CTRL_TEXT m_MaskTextBox;
		MGUI::FORM_COLORBUTTON m_IncludeColorSelect;
		MGUI::FORM_COLORBUTTON m_ExcludeColorSelect;
		MGUI::FORM_COLORBUTTON m_MarkColorSelect;
		MGUI::FORM_EDIT_NUMBER m_IncludeTransparencySelect;
		MGUI::FORM_EDIT_NUMBER m_ExcludeTransparencySelect;
		MGUI::FORM_EDIT_NUMBER m_MarkTransparencySelect;
		MGUI::FORM_SLIDER *m_RangeBeginSlider;
		MGUI::FORM_SLIDER *m_RangeEndSlider;
		MGUI::FORM_SLIDER *m_ValueRangeSlider;
		MGUI::FORM_COMBOBOX m_RangeType;
		MGUI::CTRL_PUSHBUTTON m_ThreshApplyButton;
		MGUI::CTRL_PUSHBUTTON m_ShowManualButton;
		MGUI::CTRL_TOOLBAR m_EditToolBar;
		MGUI::FORM_RADIOBUTTONS m_SetSaveMode;
		#endif
	};
} //end GRE namespace

#endif //INC_MI32_DLGMASKEDIT_H