/**
 * \file gre/lrkml.h
 * \brief GRE::LAYER_KML definitions.
 *
 * \if NODOC
 * $Id: lrkml.h_v 1.37 2012/05/08 20:50:51 mju Exp $
 *
 * $Log: lrkml.h_v $
 * Revision 1.37  2012/05/08 20:50:51  mju
 * Add dispparm::m_autoReloadInterval and method to get.
 * Remove unneeded dispparm members.
 * Add m_timeLastUpdate.
 *
 * Revision 1.36  2011/11/17 18:37:23  mju
 * Remove unused members/methods.
 * Make dispparm members private.
 * General cleanup.
 *
 * Revision 1.35  2011/10/20 13:05:03  mju
 * Remove redundant hdrs and add inclusion guards for others.
 *
 * Revision 1.34  2011/10/18 17:05:42  ldyas
 * added process placemarks
 *
 * Revision 1.33  2011/10/18 14:56:48  ldyas
 * added load styles into dispparm
 *
 * Revision 1.32  2011/10/18 14:02:31  ldyas
 * added line style to kmlelemstyle
 *
 * Revision 1.30  2011/03/02 23:07:16  ldyas
 * added show polygon fills to controls
 *
 * Revision 1.29  2010/11/19 23:09:04  bcolombe
 * Remove use of DLG_GETOBJECT::ITEM
 *
 * Revision 1.28  2010/11/05 20:45:27  ldyas
 * use kmlelemstyle for polygons and points both.
 *
 * Revision 1.27  2010/11/04 17:22:25  ldyas
 * make points scale not privat
 *
 * Revision 1.25  2010/11/02 23:06:08  ldyas
 * addedd rotation to ReadImage
 *
 * Revision 1.24  2010/10/27 21:21:01  ldyas
 * added snap to methods
 *
 * Revision 1.19  2010/09/16 16:31:13  ldyas
 * added search tree among other bits.
 *
 * Revision 1.16  2010/08/19 23:08:31  bcolombe
 * Add v_AssignItem
 *
 * Revision 1.15  2010/08/11 22:16:49  bcolombe
 * Added option to construct layer from DLG_GETOBJECT::IMTE
 *
 * Revision 1.14  2010/07/22 23:00:35  ldyas
 * added callback layer refresh
 *
 * Revision 1.8  2010/05/17 17:57:36  ldyas
 * adjusted using point styles.
 *
 * Revision 1.1  2010/05/07 17:51:58  ldyas
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_LRKML_H
#define	INC_GRE_LRKML_H

#ifndef	INC_GRE_LAYER_H
#include <gre/layer.h>
#endif

#ifndef	INC_GRE_LEGEND_H
#include <gre/legend.h>
#endif

#ifndef	INC_GRE_DATATIP_H
#include <gre/datatip.h>
#endif

#ifndef	INC_RVC_STYLEPATTERNBITMAP_H
#include <rvc/stylepatternbitmap.h>
#endif

#ifndef	INC_SPATMOD_IMAGESTAGEARRAY_H
#include <spatmod/imagestagearray.h>
#endif

#ifndef	INC_MI32_MISTRINGLIST_H
#include <mi32/mistringlist.h>
#endif

#ifndef	INC_MI32_XML_H
#include <mi32/xml.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class QUADTREE;
#endif


namespace GRE {

class LAYER_KML : public GRE::LAYER {
	public:
		 
		// LAYER_KML::DISPPARM
		class DISPPARM : public GRE::LAYER::DISPPARM {
			public:

				DISPPARM ();

				DISPPARM (
					const DISPPARM& rhs
					);

				virtual ~DISPPARM ();

				DISPPARM& operator= (
					const DISPPARM& rhs
					);

				UINT16 GetAutoReloadInterval () const
					{ return (m_AutoReloadInterval); }

				bool GetAutoUpdate (
					) const { return (m_autoupdate); }

				const DATATIPPARMS& GetDataTipParms () const
					{ return (m_DataTip); }

				const TEXTSTYLE& GetLabelStyle (
					) const { return (m_LabelStyle); }

				const RVC::OBJITEM& GetObjItem () const
					{ return (m_ObjItem); }

				const FILEPATH& GetPath (
					) const { return (m_ObjItem.GetFilePath()); }

				double GetPointScale (
					) const { return m_PointScale; }

				const POINTSTYLE& GetPointStyle (
					) const { return (m_PointStyle); }

				const POLYSTYLE& GetPolyStyle (
					) const { return (m_PolyStyle); }

				UINT16 GetSelectMode () const
					{ return (m_SelectMode); }
				
				bool GetShowLabels (
					) const { return (m_showlabels); }

				const MISTRING& GetURL (
					) const { return (m_ObjItem.GetURL()); }

				bool HasURL () const
					{ return (m_ObjItem.GetType() == RVC::OBJITEM::TYPE_URL); }

				void SetAutoUpdate (
					bool autoupdate = true
					) { m_autoupdate = autoupdate; }

				void SetDataTipParms (
					const DATATIPPARMS& DataTipParms
					) { m_DataTip = DataTipParms; }

				void SetLabelStyle (
					const TEXTSTYLE& style
					) { m_LabelStyle = style; }

				void SetObjItem (
					const RVC::OBJITEM& ObjItem
					) { m_ObjItem = ObjItem; }

				void SetPointScale (
					double scale
					) { m_PointScale = scale; }

				void SetPointStyle (
					const POINTSTYLE& style
					) { m_PointStyle = style; }

				void SetPolyStyle (
					const POLYSTYLE& style
					) { m_PolyStyle = style; }

				void SetShowLabels (
					bool label = true
					) { m_showlabels = label; }

				void SetURL (
					const MISTRING& url
					) { m_ObjItem.SetURL(url,RVC::OBJTYPE_RemoteKML); }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				typedef LAYER::DISPPARM BASECLASS;
				typedef LAYER_KML::DISPPARM THISCLASS;
		
				RVC::OBJITEM m_ObjItem;
				POLYSTYLE m_PolyStyle;
				LINESTYLE m_LineStyle;
				POINTSTYLE m_PointStyle;
				TEXTSTYLE m_LabelStyle;
				GRE::DATATIPPARMS m_DataTip;
				double m_PointScale;
				UINT16 m_SelectMode;
				UINT16 m_AutoReloadInterval;
				bool m_autoupdate;
				bool m_showlabels;
			
				void Copy (const DISPPARM&);
				void Free ();

				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;

				virtual ERRVALUE v_SetObjItem (const RVC::OBJITEM& objitem, bool replace);
				virtual ERRVALUE v_Validate ();
				
				#endif // GENERATING_DOXYGEN_OUTPUT
			};	// End of LAYER_KML::DISPPARM

		//! Register this layer type
		static void RegisterType ();

		//! Construct layer and insert into list
		explicit LAYER_KML (
			GRE::GROUP *group,										//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,		//!< Position in list
			GRE::LAYER *reflayer = 0								//!< Reference layer for insertion
			);

		//! Destructor.
		virtual ~LAYER_KML ();

		//! Retrieve layer-specific display parameters.
		const DISPPARM& GetDispParm (
			) const { return (m_dispparm); }

		//! Set layer-specific display parameters.
		//! Validation will be performed on the specified parameters and
		//! adjustments made to the layer's copy if necessary.  Use of this method
		//! usually result in a ChangeBegin / ChangeEnd message pair, and will also
		//! generate a ChangeObject message if the spatial object used in the layer
		//! is changed.
		ERRVALUE SetDispParm (
			const DISPPARM& dispparm					//!< Parameters to set
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef LAYER_KML THISCLASS;

		class DLG;

		class KMLELEMSTYLE {
			public:
				POINTSTYLE m_PointStyle;
				POLYSTYLE m_PolyStyle;
				LINESTYLE m_LineStyle;
				TEXTSTYLE m_LabelStyle;
				INT32 m_IconIndex;
				MISTRING m_Label;
				MISTRING m_Description;
				double m_LabelScale;
				bool m_ShowLabel;

				KMLELEMSTYLE ();
				~KMLELEMSTYLE ();
			};

		DISPPARM m_dispparm;
		FILEPATH m_pathkmz;								// Path to KMZ file if used, may be a temporary file if source is remote
		XMLNODE *m_RootNode;
		XMLDOC m_XmlDoc;
		POLYLINE m_Locations;
		POLYLINELIST m_Paths;
		MILIST<REGION2D> m_Areas;
		MILIST<KMLELEMSTYLE> m_PolyStyles;
		MILIST<LINESTYLE> m_LineStyles;
		MILIST<KMLELEMSTYLE> m_PointStyles;
		SPATMOD::IMAGE::STAGE_ARRAY m_GroundOverlays;
		MILIST<RVC::STYLEPATTERN_BITMAP> m_IconBitmaps;
		MISTRINGLIST m_IconUrls;
		MISTRING m_Username;
		MISTRING m_Password;
		QUADTREE *m_SearchTree;
		INT32 m_LastNumRecords;
		INT32 m_SearchNumAdded;
	#ifdef USE_MGUI
		DLG *m_pDlgControls;
	#endif
		GRE::LEGEND_POLYGON m_Legend;
		double m_TimeLastUpdate;
		bool m_IsGoogle;

		static void GRECB_Layer (const GRE::MSG *msg, void *cbdata);	
		static void LegendMsgHandler (GRAPHICTREEITEM_MSG *msg);

		ERRVALUE GetRealResourcePath (const MISTRING& src, MISTRING& actual) const;
		void ProcessGroundOverlays (SIMPLE_ARRAY<XMLNODE*>& GroundOverlays, DRECT2D& extent); 
		ERRVALUE ProcessPlacemarks (SIMPLE_ARRAY<XMLNODE*>& Places, DRECT2D& extents);
		ERRVALUE ProcessStyle (KMLELEMSTYLE& CurrentStyle, XMLNODE *node);
		ERRVALUE ReadImage(const MISTRING& src, DRECT2D& extents, SPATMOD::IMAGE::STAGE_AUTOPTR& StagePtr, double rotation = 0.0);
		ERRVALUE ReadSymbol (RVC::STYLEPATTERN_BITMAP& Bitmap, const COLOR& color, const MISTRING& src, const DPOINT2D& hotspot, const MISTRING& unitsx, const MISTRING& unitsy);
		ERRVALUE SetupSrcImageStage (const MISTRING& src, SPATMOD::IMAGE::STAGE_AUTOPTR& PtrStage) const;
		ERRVALUE Update ();
		void UpdateName ();

		// Overrides from GRE::LAYER.
		virtual ERRVALUE v_AssignObject (const RVC::OBJITEM& objitem, MDLGPARENT dlgparent);
	#ifdef USE_MGUI
	 	virtual int v_ControlPanel (MDLGPARENT dlgparent,UINT32 flags = 0);
	#endif
		virtual GRE::LAYER* v_Copy (GRE::GROUP *group) const;
		virtual const GRE::DATATIPPARMS* v_DataTipGetParms (int component) const;
		virtual int v_DataTipGetString (ELEMTYPE ElemType, ELEMNUM ElemNum, MISTRING& string, SPATMOD::IMAGE::STAGE_ARRAY& images);
		bool v_DataTipHasAvailable () const;
		bool v_DataTipHasEnabled () const;
		virtual ERRVALUE v_DataTipSetParms (int component, const GRE::DATATIPPARMS& DataTipParms);
		virtual ERRVALUE v_Draw (GRE::LAYERDC& layerdc);
		virtual ELEMNUM v_FindClosestElement (const DPOINT2D& point, double searchdist, double *distret, int component, const UNSIGNEDSET *pSearchSet, GRE::FINDFLAGS findflags);
		virtual UINT16 v_GetElemDrawSelectionMode (ELEMTYPE ElemType) const;
		virtual ERRVALUE v_GetElemExtents (ELEMTYPE ElemType, ELEMNUM ElemNum, DRECT3D& Extents);
		virtual const GRE::LEGEND* v_GetLegend (ELEMTYPE ElemType);
		virtual INT64 v_GetNumElements (ELEMTYPE ElemType);
		virtual const RVC::OBJITEM& v_GetObjItem (int component) const;
		virtual bool v_HasLegend () const;
	#ifdef USE_MGUI
	 	virtual ERRVALUE v_PromptObject (bool OpenCtrlPanel, MDLGPARENT dlgparent);
	#endif
		virtual const char* v_SerialGetTagName () const;
		virtual void v_SetLegendSampleFlags (MGD::STYLESAMPLEFLAGS SampleFlags);
		virtual int v_SnapPoint (DPOINT3D& ptObject, SNAPTO SnapTo, double MaxDistance, VIEW *view) const;
		virtual bool v_SnapPointIsAllowed (SNAPTO SnapTo) const;
		virtual int v_UpdateExtents (bool forceupdate);
		virtual bool v_UsesTransparency (const MGD::DEVICE *pDevice = 0) const;

		LAYER_KML (const LAYER_KML&);
		LAYER_KML& operator= (const LAYER_KML&);
		
		class PRIVDRAWDATA;
		friend class PRIVDRAWDATA;

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}	// End namespace GRE

#endif	// INC_GRE_LRKML_H
