/**
 * \file mgui/dlgselectlist.h
 * \brief MGUI::DLG_SELECT_LIST class definition.
 *
 * \if NODOC
 * $Id: dlgselectlist.h_v 1.5 2011/11/22 13:30:32 mju Exp $
 *
 * $Log: dlgselectlist.h_v $
 * Revision 1.5  2011/11/22 13:30:32  mju
 * Remove redundant hdrs.
 *
 * Revision 1.4  2010/01/28 16:43:36  mju
 * Fix doxygen file tag at top.
 *
 * Revision 1.3  2008/09/09 21:53:15  mju
 * Fix doxygen warning.
 *
 * Revision 1.2  2008/09/03 13:15:43  mju
 * Fix doxygen warnings.
 *
 * Revision 1.1  2007/10/17 17:56:43  scowan
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_DLGSELECTLIST_H
#define	INC_MGUI_DLGSELECTLIST_H

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef	INC_MGUI_LISTBOX_H
#include <mgui/listbox.h>
#endif

#ifndef	INC_MGUI_LISTITEM_H
#include <mgui/listitem.h>
#endif

namespace MGUI {

//! Dialog to display a list of strings to select one of.
class DLG_SELECT_LIST : public DLGSHELL {
	public:
		enum FLAGS {
			FLAG_Default = 	0x00,
			FLAG_AllowNone = 	0x01				//!< Add a \<None\> choice at the top of the list
			};

		//! Constructor.
		DLG_SELECT_LIST (
			const MISTRING& dlgtitle,				//!< Text for title
			const MISTRING& prompt,					//!< Text for prompt
			FLAGS flags = FLAG_Default,			//!< Flags for dialog
			HELPID helpid = HELPID__None			//!< Help ID
			);

		//! Destructor.
		~DLG_SELECT_LIST (
			);

		//! Add an item to the list, must be used before the call to DoModal().
		void AddItem (
			MGUI::ID id,						//!< MGUI::ID to assign to the list item
			const MISTRING& string			//!< The string to show in the list
			);
			
		//! Get the selected item MGUI::ID.
		//! If the dialog is created using FLAG_AllowNone and the user selects \<None\>, the return value will be EUserCancel
		//! @return MGUI::ID of the selected item in the list.
		MGUI::ID GetSelectedItemID (
			);

		//! Set the default item in the selection list using the MGUI::ID given in AddItem().
		void SetDefaultItemID (
			MGUI::ID id
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MILIST<MGUI::LISTITEM> m_ItemList;
		CTRL_LABEL m_PromptLabel;
		CTRL_LISTBOX m_ListBox;
		MGUI::ID m_DefaultID;
		MISTRING m_PromptString;
		FLAGS m_flags;

		void OnDoubleClick ();
		
		virtual ERRVALUE v_CreateContent ();
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


}	// End namespace MGUI

#endif	// INC_MGUI_DLGSELECTLIST_H
