/**
 * \file mgui/dlgtextv.h
 * \brief MGUI::DLG_TEXTVIEW class definitions.
 *
 * \if NODOC
 * $Id: dlgtextv.h_v 1.10 2011/09/01 17:54:43 mju Exp $
 *
 * $Log: dlgtextv.h_v $
 * Revision 1.10  2011/09/01 17:54:43  mju
 * Exclude macro from dox.
 *
 * Revision 1.9  2006/02/01 16:06:49  dwilliss
 * Added optional STYLE parameter
 *
 * Revision 1.8  2005/06/27 13:34:52  mju
 * Include edit hdr.
 *
 * Revision 1.7  2005/03/31 16:57:08  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.6  2003/11/05 21:11:58  mju
 * Add default for helpid.
 *
 * Revision 1.5  2003/10/03 19:58:19  linux32build!build
 * Doxygen
 *
 * Revision 1.4  2003/09/23 14:20:05  scowan
 * nc.
 *
 * Revision 1.3  2003/09/22 19:53:56  scowan
 * Changed to use dlg shell instead of dlg base.
 *
 * Revision 1.2  2003/09/15 13:49:32  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.1  2002/10/23 15:39:25  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_DLGTEXTV_H
#define	INC_MGUI_DLGTEXTV_H

#ifndef INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif

namespace MGUI {
//===================================================================================================================
//! Dialog for viewing non-editable text.
class DLG_TEXTVIEW : public DLGSHELL {
	public:

		enum STYLE {
			STYLE_Normal = 				0x00000000,
			STYLE_UseFixedWidthFont = 	0x00000001
			};

		//! Constructor.
		DLG_TEXTVIEW (
			const MISTRING& title,						//!< Title string, can use TEXTID also
			HELPID HelpID = HELPID__None,				//!< Help ID
			STYLE style = STYLE_Normal
			);

		//! Destructor.
		virtual ~DLG_TEXTVIEW ();

		//! Set text to display from MISTRING.
		void SetText (
			const MISTRING& string
			);

		//! Set text to display from MIUNICODE string.
		void SetText (
			const MIUNICODE *string
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MISTRING m_TextStr;
		CTRL_EDIT_TEXT m_TextCtrl;
		STYLE m_style;

		//! Overrides from DLGSHELL.
		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnOK ();
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DLG_TEXTVIEW::STYLE)
#endif

//===================================================================================================================
}	// End namespace MGUI

#endif	// INC_MGUI_DLGTEXTV_H
