/**
 * \file mgui/smlhelp.h
 * \brief SML / Query / Script Editor class
 *
 * \if NODOC
 * $Log: smlhelp.h_v $
 * Revision 1.15  2008/10/31 20:30:30  scowan
 * Replaced legacy status classes.
 *
 * Revision 1.14  2007/11/01 22:25:53  bcolombe
 * Remove TEST_HTML ifdefs from around FORM_HTML
 *
 * Revision 1.13  2007/07/25 21:10:48  dwilliss
 * use push buttons for MFC because toolbar not implemented yet
 *
 * Revision 1.12  2007/07/23 19:13:36  dwilliss
 * added a find label.
 * Started implementing HTML, but ifdef'd out
 *
 * Revision 1.11  2007/05/18 19:46:16  dwilliss
 * added usage parameters
 *
 * Revision 1.10  2007/05/09 19:25:03  dwilliss
 * fixed drb's fix
 *
 * Revision 1.9  2007/05/09 19:20:59  drb
 * Fixed some sml namespace issues
 *
 * Revision 1.8  2007/05/07 14:21:44  mju
 * Use mistringlist hdr.
 *
 * Revision 1.7  2007/02/05 16:05:10  dwilliss
 * critical section is now in mithread namespace
 *
 * Revision 1.6  2007/02/05 15:56:49  scowan
 * Fixed forward declaration.
 *
 * Revision 1.5  2007/02/01 15:18:20  dwilliss
 * added some members
 *
 * Revision 1.4  2007/01/17 21:28:03  dwilliss
 * Added a private method
 *
 * Revision 1.3  2007/01/17 20:41:24  dwilliss
 * *** empty log message ***
 *
 * Revision 1.2  2007/01/17 16:41:34  dwilliss
 * Added new section enum
 *
 * Revision 1.1  2006/10/30 23:50:26  dwilliss
 * Initial revision
 *
 * \endif
 **/

#ifndef INC_MGUI_SMLHELP_H
#define INC_MGUI_SMLHELP_H

#ifndef	INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef	INC_MGUI_SASH_H
#include <mgui/sash.h>
#endif

#ifndef	INC_MGUI_TREE_H
#include <mgui/tree.h>
#endif

#ifndef	INC_MGUI_TOOLBAR_H
#include <mgui/toolbar.h>
#endif

#ifndef	INC_MGUI_TIMER_H
#include <mgui/timer.h>
#endif

#ifndef	INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif

#ifndef	INC_MI32_XML_H
#include <mi32/xml.h>
#endif

#ifndef	INC_MI32_MISTRINGLIST_H
#include <mi32/mistringlist.h>
#endif

#ifndef	INC_MGUI_HTML_H
#include <mgui/html.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
namespace SML {
	class CONTEXT;
	class DOCCODES;
	}
class SMLDOCCODES;
class MISTATUSCONTEXT;
namespace MITHREAD {
	class CRITICALSECTION;
	}
#endif

namespace MGUI {

	//! Window for SML help
	class DLG_SML_HELP : public MGUI::DLGSHELL {
		public:

			enum SECTION {
				SECTION_Constants,
				SECTION_Fields,
				SECTION_Variables,
				SECTION_Functions,
				SECTION_Classes,
				SECTION_Keywords,
				SECTION_Operators,
				SECTION_Examples
				};

			class SEARCHRESULT {
				public:
					SEARCHRESULT (
						) :
						score(0)
						{ }

					SEARCHRESULT (
						const SEARCHRESULT& rhs
						) :
						name(rhs.name),
						score(rhs.score)
						{ }

					~SEARCHRESULT() {}

					bool operator<(const SEARCHRESULT&rhs);
					
					MISTRING name;
					INT32 score;
				};

			DLG_SML_HELP (
				SML::CONTEXT* context,
				int segment
				);

			virtual ~DLG_SML_HELP ();

			void SetDelegateOnInsert (
				DELEGATE_VOID_MISTRING delegate
				) { m_DelegateInsert = delegate; }

			void JumpToSection (
				SECTION section
				);

			bool SearchItem (
				INT32 id,
				SEARCHRESULT& result,
				const MISTRING& SearchString
				) const;

#ifndef GENERATING_DOXYGEN_OUTPUT
#ifdef WIN32 // A work in progress...
			void SearchProc (
				int start,
				int end,
				int interval,
				const MISTRING& SearchStr,
				MILIST<SEARCHRESULT>& Results,
				MITHREAD::CRITICALSECTION& cs,
				MISTATUSCONTEXT* StatusContext
				);
#endif
#endif

		private:
#ifndef GENERATING_DOXYGEN_OUTPUT
			enum BUTTON {
				BUTTON_NavigateBack,
				BUTTON_NavigateForward,
				BUTTON_Insert,
				BUTTON_InsertExample,
				BUTTON_Close
				};

			class MYTREE : public MGUI::CTRL_TREE {
				public:
					MYTREE (
						) :
						m_dlg(0) 
						{}

					void SetDlg (
						DLG_SML_HELP* dlg
						) { m_dlg = dlg; }

				private:

					virtual bool v_OnChildVisibilityChanging (
						ITEMHND itemHnd,
						CHILDVISIBILITY visibility
						);

					virtual void v_OnItemSelected (
						ITEMHND hdnItem
						);

					virtual bool v_OnToolTipRequest (
						ITEMHND hndItem,
						HITRESULT,
						const LPOINT2D&,
						MGUI::CTRL_TOOLTIP&
						);

					virtual MGUI::CUSTOMDRAW::RETVALUE v_OnCustomDraw (
						MGUI::CUSTOMDRAW_TREE& CustomDraw
						);

					DLG_SML_HELP* m_dlg;
				};

			enum ITEMTYPE {
				ITEMTYPE_ConstFolder,
				ITEMTYPE_Const,
				ITEMTYPE_VariableFolder,
				ITEMTYPE_Variable,
				ITEMTYPE_FieldFolder,	// XXX hierarchy?
				ITEMTYPE_Field,
				ITEMTYPE_FuncFolder,
				ITEMTYPE_FuncGroup,
				ITEMTYPE_Func,
				ITEMTYPE_ClassFolder,
				ITEMTYPE_Class,
				ITEMTYPE_OperatorFolder,
				ITEMTYPE_Operator,
				ITEMTYPE_KeywordFolder,
				ITEMTYPE_Keyword,
				ITEMTYPE_SearchResult,
				ITEMTYPE_ExamplesFolder,
				ITEMTYPE_Example
				};

			struct MYITEM {
				ITEMTYPE type;
				int num;
				const void* ptr;	// what it points to depends on type
				MGUI::CTRL_TREE::ITEMHND hnd;

				static int CompareChar(MYITEM*, MYITEM*, void*);
				};

#ifdef X_NATIVE
			MGUI::CTRL_TOOLBAR m_ToolBar;
#else
			MGUI::CTRL_PUSHBUTTON m_BtnNavigateBack;
			MGUI::CTRL_PUSHBUTTON m_BtnNavigateForward;
			MGUI::CTRL_PUSHBUTTON m_BtnInsert;
			MGUI::CTRL_PUSHBUTTON m_BtnInsertExample;
#endif
			MYTREE m_Tree;
			MGUI::CTRL_SASH m_Sash;
			MGUI::CTRL_EDIT_TEXT m_Text;
			MGUI::FORM_HTML m_HTMLCtrl;
			SML::CONTEXT* m_context;
			int m_segment;
			DELEGATE_VOID_MISTRING m_DelegateInsert;
			MGUI::CTRL_EDIT_STRING m_EditSearch;
#ifdef X_NATIVE
			MGUI::CTRL_TOOLBAR m_SearchToolBar;
#else
			MGUI::CTRL_PUSHBUTTON m_BtnSearch;
#endif
			MGUI::TIMER m_Timer;
			MISTRING m_QuickSearchValue;
			MGUI::CTRL_LABEL m_FindLabel;
			
			virtual ERRVALUE v_CreateContent();
			SIMPLE_ARRAY<MYITEM> m_Items;
			SIMPLE_ARRAY<INT32> m_NavStack;
			INT32 m_NavIndex;
			MISTRINGLIST m_SearchResults;
			MGUI::CTRL_TREE::ITEMHND m_ItemHndExamplesFolder;
			bool m_bExamplesAdded;
			XMLDOC m_ClassGroupsDoc;
			SIMPLE_ARRAY<INT32> m_ClassGroupItems;

			MGUI::CTRL_TREE::ITEMHND AddItem (
				MGUI::CTRL_TREE::ITEMHND parent,
				ITEMTYPE type,
				const MISTRING& name,
				MGUI::CTRL_TREE::CHILDPRESENCE = MGUI::CTRL_TREE::CHILDPRESENCE_No,
				int num = 0,
				const void* ptr = 0,
				MGUI::CTRL_TREE::ITEMHND after = MGUI::CTRL_TREE::ITEMHND_Last
				);

			INT32 FindItemID (
				const MISTRING& name,
				MGUI::CTRL_TREE::ITEMHND parent = MGUI::CTRL_TREE::ITEMHND_Root
				);

#ifdef X_NATIVE
			void OnNavigateBack(MGUI::ID, bool);
			void OnNavigateForward(MGUI::ID, bool);
			void OnBtnInsert(MGUI::ID, bool);
			void OnBtnInsertExample(MGUI::ID, bool);
#else
			void OnNavigateBack();
			void OnNavigateForward();
			void OnBtnInsert();
			void OnBtnInsertExample();
#endif

			void OnBtnClose(MGUI::ID, bool) { Close(); }
			void OnHyperLink(const MISTRING& href);
			void OnBtnSearch(MGUI::ID, bool);
			void OnSearch();	
			void OnQuickSearch();	
			void OnEditSearch();	

			void ShowItemHelp (INT32 id, bool push);
			ERRVALUE GenerateItemHelp (INT32 id, SML::DOCCODES& doc, bool bFull=true) const;
			void UpdateVariableList();
			void AddExampleNodes();

			void UpdateTreeList (
				MGUI::CTRL_TREE::ITEMHND folder,
				const MISTRINGLIST& names,
				ITEMTYPE type
				);

			friend class MYTREE;
#endif
		};


}	// End namespace MGUI


#endif	// INC_MGUI_SMLHELP_H

