/**
 * \file mi32/mg2mem1.h
 * \brief MGD::DEVICE_MEM1 definitions.
 *
 * \if NODOC
 * $Log: mg2mem1.h_v $
 * Revision 1.17  2011/11/11 14:30:25  mju
 * Make context nested and don't inline virtuals.
 *
 * Revision 1.6  2002/04/18 19:58:21  dwilliss
 * Added a Copy method
 *
 * Revision 1.1  2002/01/22 18:00:58  scowan
 * Initial revision
 * \endif
**/
 
#ifndef INC_MI32_MG2MEM1_H
#define INC_MI32_MG2MEM1_H

#ifndef INC_MI32_MG2BINARY_H
#include <mi32/mg2binary.h>
#endif

#ifndef INC_MI32_BITSET_H
#include <mi32/bitset.h>
#endif


namespace MGD {

//! Memory-based implementation of MGD::DEVICE for 1-bit visuals
//!
//! This version is implemented with a BITSET_SHARED, and is most efficient
//! for small things that will fit in memory well.  For larger devices, use
//! MGD::DEVICE_MEM1_SPARSE
class DEVICE_MEM1 : public MGD::DEVICE_BINARY {
	public:

		class CONTEXT : public MGD::DEVICE_BINARY::CONTEXT {
			public:
				//! Destructor.
				virtual ~CONTEXT ();

			protected:
				//! Default constructor.
				CONTEXT (
					MGD::DEVICE_MEM1 *pDevice
					);

			private:
		   	#ifndef GENERATING_DOXYGEN_OUTPUT
				typedef DEVICE_BINARY::CONTEXT BASECLASS;

				CONTEXT (const CONTEXT&);
				CONTEXT& operator= (const CONTEXT&);

				MGD::DEVICE_MEM1 *m_DevMem1;

				friend class MGD::DEVICE_MEM1;
				
				virtual ERRVALUE v_PutPixel (INT32 x, INT32 y, MGD::PIXEL pixel);
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Constructor.
		DEVICE_MEM1 ();

		//! Destructor.
		virtual ~DEVICE_MEM1 ();

		//! Copy a DEVICE_MEM1.
		void Copy (
			const DEVICE_MEM1& rhs
			) {
			if (&rhs != this) {
				m_width = rhs.m_width;
				m_buf = rhs.m_buf;
				}
			return;
			}

		ERRVALUE Create (
			INT32 height,
			INT32 width,
			bool StartValue = false
			);

		void CombineMasks (
			const BITSET_UNOWNED& rhs,
			PIXELFUNC PixelFunc
			);
			
		const BITSET_SHARED& GetImage (
			) const { return (m_buf); }
			
		void SetImage (
			const BITSET_SHARED& image
			) { m_buf = image; }
			
		void SetImage (
			const BITSET_UNOWNED& image
			) { m_buf = image; }
			
	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		DEVICE_MEM1 (const DEVICE_MEM1&);
		DEVICE_MEM1& operator= (const DEVICE_MEM1&);

		BITSET_SHARED m_buf;
		UINT32 m_width;
		
		// DEVICE overrides.
		virtual MGD::CONTEXT* v_CreateContext ();
		virtual ERRVALUE v_FillSpan (MGD::CONTEXT* vgc, INT32 x1, INT32 x2, INT32 y);
		// DEVICE_BINARY overrides.
		virtual bool v_GetRange (INT32 x, INT32 y, INT32 x2, INT32& endx) const;
		virtual bool v_GetValue (INT32 x, INT32 y) const;
		virtual void v_SetAll (bool value);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}	// End of namespace

#endif	// INC_MI32_MG2MEM1_H

