/******************************************************************************
 * \file mi32/mg2mem32rgb.h
 * \brief MGD::DEVICE_MEM32RGB implementation.
 *
 * \if NODOC
 * $Id: mg2mem32rgb.h_v 1.1 2012/02/20 18:40:03 mju Exp $
 *
 * $Log: mg2mem32rgb.h_v $
 * Revision 1.1  2012/02/20 18:40:03  mju
 * Initial revision
 *
 * \endif
 */

#ifndef INC_MI32_MG2MEM32RGB_H
#define INC_MI32_MG2MEM32RGB_H

#ifndef INC_MI32_MG2MEM32_H
#include <mi32/mg2mem32.h>
#endif


namespace MGD {

//! Memory-based drawing device for 32-bit RGB visuals.
class DEVICE_MEM32RGB : public MGD::DEVICE_MEM32 {

	public:

		//! constructor
		DEVICE_MEM32RGB ();

		//! Destructor
		virtual ~DEVICE_MEM32RGB ();

		//! Create device with single frame buffer.
		//! If the ColorSpace is provided, it must be one of the following:
		//!	COLORSPACE_X8_R8_G8_B8 (the default), 
		//!	COLORSPACE_R8_G8_B8_X8
		//!	COLORSPACE_X8_B8_G8_R8
		//!	COLORSPACE_B8_G8_R8_X8
		//! Any other colorspace will return an error.
		ERRVALUE Create (
			INT32 height,							//!< Height (number of rows) in pixels
			INT32 width,							//!< Width (number of columns) in pixels
			COLORSPACE ColorSpace = COLORSPACE_X8_R8_G8_B8,
			FORMAT format = FORMAT_Default	//!< Memory format
			) { return (Create(height,width,1,ColorSpace,format)); }

		//! Create device with option for multiple frame buffers.
		//! If the ColorSpace is provided, it must be one of the following:
		//!	COLORSPACE_X8_R8_G8_B8 (the default), 
		//!	COLORSPACE_R8_G8_B8_X8
		//!	COLORSPACE_X8_B8_G8_R8
		//!	COLORSPACE_B8_G8_R8_X8
		//! Any other colorspace will return an error.
		ERRVALUE Create (
			INT32 height,							//!< Height (number of rows) in pixels
			INT32 width,							//!< Width (number of columns) in pixels
			UINT16 NumFrameBuffers,				//!< Number of frame buffers
			COLORSPACE ColorSpace = COLORSPACE_X8_R8_G8_B8,
			FORMAT format = FORMAT_Default	//!< Memory format
			);

	protected:

		//! Context for 32-bit RGB memory device.
		class CONTEXT : public MGD::DEVICE_MEM32::CONTEXT {
			public:

				virtual ~CONTEXT ();

			protected:

				//! Default Constructor 
				CONTEXT (
					MGD::DEVICE_MEM32RGB *pDevice
					);

			private:
		   	#ifndef GENERATING_DOXYGEN_OUTPUT

				// Unimplemented
				CONTEXT (const CONTEXT&);
				CONTEXT& operator= (const CONTEXT&);

				friend class MGD::DEVICE_MEM32RGB;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT

		// Unimplemented
		DEVICE_MEM32RGB (const DEVICE_MEM32RGB&);
		DEVICE_MEM32RGB& operator= (const DEVICE_MEM32RGB&);

		virtual MGD::CONTEXT* v_CreateContext ();

		friend class MGD::CONTEXT;
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

		
}	// End of namespace

#endif	// INC_MI32_MG2MEM32RGB_H

