/************************************************************************
*
* mi32/mstyle.h - CartoScript functions
*
* Line Style Toolkit 
* \if NODOC
* $Id: mstyle.h_v 1.24 2004/02/12 21:57:26 dwilliss Exp $
* $Log: mstyle.h_v $
* Revision 1.24  2004/02/12 21:57:26  dwilliss
* Don't include deprecated include file that we didn't even need
*
* Revision 1.23  2003/09/15 13:49:56  fileserver!dwilliss
* Doxygen
*
* Revision 1.22  2003/08/11 16:31:32  dwilliss
* Took out the last 4 functions that shouldn't be called by anything but
* the cartoscript module
*
* Revision 1.21  2003/08/11 14:40:06  dwilliss
* Most of the functions are now private to the SML module that implements
* cartoscript since it was the only thing that ever called them.  Their
* prototypes are no longer here.
*
* Revision 1.20  2003/06/16 16:36:15  scowan
* Added forward declaration.
*
* Revision 1.19  2003/06/14 02:10:10  scowan
* Chagned use of dregion to a trans2d mapgen.
*
* Revision 1.18  2002/04/26 22:19:27  dwilliss
* MstyleInit should take CONST pointer for poitns
*
* Revision 1.17  2000/06/27 20:08:17  sparsons
* Genitor documentation.
*
* Revision 1.16  1999/10/21 23:15:46  scowan
* More const stuff.
*
* Revision 1.15  1999/09/27 19:13:14  dwilliss
* Added prototypes for UC versions of text drawing functions
*
* Revision 1.14  1999/04/08 19:06:27  mju
* Add inclusion guards.
*
 * Revision 1.13  1999/01/05  20:39:46  scowan
 * *** empty log message ***
 *
* Revision 1.12  1998/04/28 19:54:18  dfrolov
* move position declaration from lbplace.c
*
 * Revision 1.11  1998/04/28  17:38:52  dfrolov
 * add new functions to the label optimization toolkit
 *
 * Revision 1.10  1998/03/20  20:50:22  dwilliss
 * Added extern "C" stuff
 *
 * Revision 1.9  1997/10/17  15:33:44  dfrolov
 * add new functions
 *
 * Revision 1.8  1997/10/08  18:29:09  dfrolov
 * add MstyleScaleOffset function
 *
 * Revision 1.7  1997/10/01  22:02:40  dfrolov
 * change declaration of MlineInit()
 *
 * Revision 1.6  1997/10/01  21:34:35  dfrolov
 * add more function prototypes
 *
 * Revision 1.5  1997/08/27  15:18:49  dfrolov
 * *** empty log message ***
 *
 * Revision 1.4  1997/08/25  20:15:37  dfrolov
 * add MlabelPlace... functions
 *
 * Revision 1.3  1997/08/12  21:22:35  dfrolov
 * add some more functions
 *
 * Revision 1.2  1997/08/10  21:28:19  dfrolov
 * some changes in function calls
 *
 * Revision 1.1  1997/08/03  22:05:25  dfrolov
 * Initial revision
 *
* \endif
*************************************************************************/

#ifndef	INC_MI32_MSTYLE_H
#define	INC_MI32_MSTYLE_H

#ifndef GENERATING_DOXYGEN_OUTPUT
class TRANS2D_MAPGEN;
#endif //!< GENERATING_DOXYGEN_OUTPUT

#define MLINE_FEATURE_NEXTVERTEX		1
#define MLINE_FEATURE_PREVVERTEX		2
#define MLINE_FEATURE_LINEEND			3
#define MLINE_FEATURE_LINEBEGIN		4

typedef void *MLINEHANDLE;
typedef void *LABELHANDLE;

/*************************************************************************

	Label positions relative to center point (+):

	[2] [1]		 [6]
	   +		  [7]+[5] , position 9 - 16 are similar to 1- 8 but shifted 0.25 width/height
	[4] [3]		 [8]

	Position 17 is right in the center of the rectangle

*************************************************************************/

#define POSITION_DELETE	0
#define POSITION_1		1
#define POSITION_2		2
#define POSITION_3		3
#define POSITION_4		4
#define POSITION_5		5
#define POSITION_6		6
#define POSITION_7		7
#define POSITION_8		8
#define POSITION_9		9
#define POSITION_10		10
#define POSITION_11		11
#define POSITION_12		12
#define POSITION_13		13
#define POSITION_14		14
#define POSITION_15		15
#define POSITION_16		16
#define POSITION_17		17
#define POSITION_LAST	POSITION_17

#define POSITION_CENTER			POSITION_17
#define POSITION_LEFTBOTTOM	POSITION_1
#define POSITION_RIGHTBOTTOM	POSITION_2
#define POSITION_LEFTUP			POSITION_3
#define POSITION_RIGHTUP		POSITION_4
#define POSITION_CENTERLEFT	POSITION_5
#define POSITION_CENTERBOTTOM	POSITION_6
#define POSITION_CENTERRIGHT	POSITION_7
#define POSITION_CENTERUP		POSITION_8

#if defined(__cplusplus)
extern "C" {
#endif

//!:Associate with "Line Style Functions"
//!\addtogroup Cartoscript Cartoscript Functions
//!@{

// Label placement optimization routines

//! Initialize label placement toolkit.
//!
//! Explanation of the label placement toolkit basics:
//!	Label placement toolkit deals with the rectangles that represent
//!	some features that should be arranged with minimum number of collisions.
//!	There are 17 fixed positions arround the points:
//!	Label positions relative to center point (+):	
//!	[2] [1]		 	[6]
//!	   +	       [7]+[5] and 8 additional positions are similar to first 8 but shifted on 0.25 of width/height
//!	[4] [3]		 	[8]
//!	It's kind of hard to draw a picture of positions 9-16 using characters.
//!	Position 17 is when point is right in the center of the rectangle.
//!	Each position has preferability factor assigned to it, so each rectangle
//!	will try to occupy position with highest preferablity factor and minimum
//!	number of collisions. Use MlabelPlaceGet/SetPositionFactors() to get/set
//!	preferability levels to positions.
//!	Initially preferability factor is equal to position number, i.e. position
//!	number 1 is most desirable.
int MlabelPlaceInit (
	LABELHANDLE *handle,						//!< Handle returned
	const DRECT2D *space,					//!< Bounding rectangle of input labels
	INT32 NumLabelsInit						//!< Initial number od labels to optimize
	);

//! Add rectangular area (or label) to optimizer. 
//!
//! Based on some tests it's not recomended to use continious variables as a rank value. 
//! Use categorical variables instead. For example: using population for	ranking cities produces 
//! worst results compared to using categorical cities rank derived from population.
//!	Example of Categorical Variable:	  
//!	  if (population < 1000) rank = 1
//!	  if (population > 1000 && population < 10000) rank = 2
//!	  if (population > 10000) rank = 3
int MlabelPlaceAddRect (
	LABELHANDLE handle,						//!< Handle returned by MlabelPlaceInit
	DRECT2D *Label,							//!< Rectangle that defines area covered by label
	INT32 ElemNum,								//!< Some unique ID-number for label
	DOUBLE Rank									//!< Label's rank  (any number), the greater is this rank - more important is this label.
	);

//! Add rectangular area (or label) to optimizer.
//!
//! "Anchor" point is used to determine an internal position of rectangle around attachment point. 
//! The closest internal position will be used, so Anchor point that you pass will not be a real 
//! attachment point in all 100% of possible cases. If you Anchor point is situated at one of
//!	the 17 possible positions, it will be equal to attachment point of the
//!	label. For more info see MlabelPlaceInit() and MlabelPlaceAddRectAtPosition().
//!	
//! Based on some tests it's not recomended to use continious variables as a rank value. 
//! Use categorical variables instead. For example: using population for
//!	ranking cities produces worst results compared to using categorical cities rank derived 
//! from population.
//!	Example of Categorical Variable:	  
//!	  if (population < 1000) rank = 1
//!	  if (population > 1000 && population < 10000) rank = 2
//!	  if (population > 10000) rank = 3
int MlabelPlaceAddRectAtAnchor (
	LABELHANDLE handle,						//!< Handle returned by MlabelPlaceInit
	DRECT2D *Label,							//!< Rectangle that defines area covered by label
	INT32 ElemNum,								//!< Some unique ID-number for label
	DOUBLE Rank,								//!< Label's rank (any number), the greater is this rank - more important is this label
	DPOINT2D *Anchor							//!< "Attachment" point of the rectangle
	);

//! Add rectangular area (or label) to optimizer.
//!
//! PosNum defines relationship between rectangle that defines a label and "attachment" point. 
//! Rectangle will be moved arround attachment point at predefined positions.
//! For example: POSITION_1 is a bottom left corner of the rectangle,
//!	POSITION_17 or POSITION_CENTER is a center of the rectangle and etc.
//!	See mstyle.h for the list of positions and constants.
//!
//! Based on some tests it's not recomended to use continious variables as a rank value. 
//! Use categorical variables instead. For example: using population for
//!	ranking cities produces worst results compared to using categorical cities rank derived 
//! from population.
//!	Example of Categorical Variable:	  
//!	  if (population < 1000) rank = 1
//!	  if (population > 1000 && population < 10000) rank = 2
//!	  if (population > 10000) rank = 3
int MlabelPlaceAddRectAtPosition (
	LABELHANDLE handle,						//!< Handle returned by MlabelPlaceInit
	DRECT2D *Label,							//!< Rectangle that defines area covered by label
	INT32 ElemNum,								//!< Some unique ID-number for label
	DOUBLE Rank,								//!< Label's rank  (any number), the greater is this rank - more important is this label.
	int PosIndex								//!< Initial position number (see MlabelPlaceInit())
	);

//! Run optimization procedure.
//!
//! NOTE: This procedure in it's current implementation is always doing some amount of optimization 
//! even if DoOptimize == FALSE. When DoOptimize is TRUE full optimization is invoked, that might
//!	be time consuming process depending on number of input labels.
int MlabelPlaceOptimize (
	LABELHANDLE handle,						//!< Handle returned by MlabelPlaceInit
	int DoOptimize,							//!< Flag for doing full optimization
	int DoDelete,								//!< Flag for deleting overlaping labels
	INT32 *NumObscured						//!< Number of labels deleted by the process returned
	);

//! Retrieve labels position.
//!
//! @return  TRUE if label is not discarded during optimization, rect contains label's position, FALSE if label was removed, error code < 0 on error.
int MlabelPlaceGetLabelPosition (
	LABELHANDLE handle,						//!< Handle returned by MlabelPlaceInit
	INT32 ElemNum,								//!< Label's unique ID number
	DRECT2D *rect								//!< Label's position
	);

//! Free resources associated with optimizer.
int MlabelPlaceFree (
	LABELHANDLE handle 						//!< Handle returned by MlabelPlaceInit
	);

//! Get position "preferability" factors.
//!
//! This functions fills out PosWeight array with internal "preferability"
//!	factors. Programmer is responsible for allocating PosWeight array.
int MlabelPlaceGetPositionFactors (
	LABELHANDLE handle,						//!< Handle returned by MlabelPlaceInit
	DOUBLE *PosWeight,						//!< Array of position "preferability" factors, all numbers 
													//! should be positive. The smaller is a preferability factor 
													//! designate more desirible position of the label.
	UINT8 startentry,							//!< Starting internal position number 1...POSITION_LAST
	UINT8 numentries							//!< Number of elements to get, number of entries in PosWeight[]
	);

//! Set position "preferability" factors.
int MlabelPlaceSetPositionFactors (
	LABELHANDLE handle,						//!< handle returned by MlabelPlaceInit
	DOUBLE *PosWeight,						//!< Array of position "preferability" factors, all numbers 
													//! should be positive. The smaller is a preferability factor 
													//! designate more desirable position of the label.
	UINT8 startentry,							//!< Starting internal position number 1...POSITION_LAST
	UINT8 numentries							//!< Number of elements to set, number of entries in PosWeight[]
	);

//!@}

#if defined(__cplusplus)
}								
#endif

#endif	//!<  INC_MI32_MSTYLE_H 
