/**
 * \file mi32/oldprojparm.h
 * \brief Definitions for legacy projection parameter structure
 *
 * \if NODOC
 * $Id: oldprojparm.h_v 1.12 2005/12/14 05:19:14 dfriberg Exp $
 *
 * $Log: oldprojparm.h_v $
 * Revision 1.12  2005/12/14 05:19:14  dfriberg
 * Add gda94 datum
 *
 * Revision 1.11  2005/09/20 17:27:19  dfriberg
 * Added more defines
 *
 * Revision 1.10  2005/09/19 17:31:38  dfriberg
 * Added OLD_PROJSYSTEM_UserDefined
 *
 * Revision 1.9  2005/09/07 17:13:46  mju
 * Add legacy values for ups and nzmg.
 *
 * Revision 1.8  2005/07/22 17:34:34  dfriberg
 * Added more old datums
 *
 * Revision 1.7  2005/07/15 14:18:23  mju
 * Include memory hdr instead of string hdr.
 *
 * Revision 1.6  2005/03/31 16:57:19  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.5  2005/01/11 22:32:26  mju
 * Add old_projection_undefined.
 *
 * Revision 1.4  2004/07/23 14:42:08  mju
 * Add old defns for utm and stateplane.
 *
 * Revision 1.3  2004/06/30 20:09:23  mju
 * Add default ctor.
 *
 * Revision 1.2  2004/06/18 20:44:02  mju
 * Add old_projsys_utm.
 *
 * Revision 1.1  2004/06/18 15:49:09  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_OLDPROJPARM_H
#define	INC_MI32_OLDPROJPARM_H

#ifndef	INC_MEMORY_H
#include <memory.h>		// Includes defn for memset()
#define	INC_MEMORY_H
#endif

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

enum OLD_PROJECTION {
	OLD_PROJECTION_Undefined =								-3,	//!< Undefined/Unspecified
	OLD_PROJECTION_Local =									-1,	//!< Arbitrary local coordinates
	OLD_PROJECTION_Geographic =							 0,	//!< Latitude/longitude
	OLD_PROJECTION_UTM =										 1,	//!< Universal Transverse Mercator (system)
	OLD_PROJECTION_StatePlane =							 2,	//!< US State Plane, system based on datumconv
	OLD_PROJECTION_AlbersEqualAreaConic =				 3,	//!< Albers Equal-Area Conic
	OLD_PROJECTION_LambertConformalConic =				 4,	//!< Lambert Conformal Conic
	OLD_PROJECTION_Mercator =								 5,	//!< Mercator
	OLD_PROJECTION_PolarStereographic =					 6,	//!< Polar Stereographic
	OLD_PROJECTION_Polyconic =								 7,	//!< Polyconic
	OLD_PROJECTION_EquidistantConic =					 8,	//!< Equidistant Conic
	OLD_PROJECTION_TransverseMercator =					 9,	//!< Transverse Mercator
	OLD_PROJECTION_Stereographic =						10,	//!< Stereographic
	OLD_PROJECTION_LambertAzimuthalEqualArea =		11,	//!< Lambert Azimuthal Equal-Area
	OLD_PROJECTION_AzimuthalEquidistant =				12,	//!< Azimuthal Equidistant
	OLD_PROJECTION_Gnomonic =								13,	//!< Gnomonic
	OLD_PROJECTION_Orthographic =							14,	//!< Orthographic
	OLD_PROJECTION_GeneralVertNearSidePerspective =	15,	//!< General Vertical Near-Side Perspective
	OLD_PROJECTION_Sinusoidal =							16,	//!< Sinusoidal
	OLD_PROJECTION_Equirectangular =						17,	//!< Equirectangular
	OLD_PROJECTION_MillerCylindrical =					18,	//!< Miller Cylindrical
	OLD_PROJECTION_VanDerGrintenI =						19,	//!< Van Der Grinten I
	OLD_PROJECTION_ObliqueMercator =						20,	//!< Oblique Mercator (Hotine)
	OLD_PROJECTION_Robinson =								21,	//!< Robinson
	OLD_PROJECTION_BipolarObliqueConformalConic =	22,	//!< Bipolar Oblique Conformal Conic
	OLD_PROJECTION_LandsatSpaceObliqueMercator =		23,	//!< Space Oblique Mercator for Landsat
	OLD_PROJECTION_Cassini =								24,	//!< Cassini
	OLD_PROJECTION_NewZealandMapGrid =					25,	//!< New Zealand Map Grid
	OLD_PROJECTION_SwissConformalCylindrical =		26,	//!< Swiss Conformal Cylindrical
	OLD_PROJECTION_RoyalDutch =							27,	//!< Dutch National Grid (RD)
	OLD_PROJECTION_ObliqueStereographic =				28,	//!< Oblique Stereographic
	OLD_PROJECTION_InterruptedGoodeHomolosine =		29,	//!< Interrupted Goode Homolosine
	};

#define	OLD_PROJSYSTEM_UserDefined		0
#define	OLD_PROJSYSTEM_LatLon			1
#define	OLD_PROJSYSTEM_StatePlane1927	3
#define	OLD_PROJSYSTEM_StatePlane1983	4
#define	OLD_PROJSYSTEM_UTM				5
#define	OLD_PROJSYSTEM_UPS				6
#define	OLD_PROJSYSTEM_NZMG				9

#define	OLD_DATUMCONV_Unspecified		0
#define	OLD_DATUMCONV_WGS84				1
#define	OLD_DATUMCONV_WGS72				2
#define	OLD_DATUMCONV_GDA94				288
#define	OLD_DATUMCONV_NewZealand1949	347
#define	OLD_DATUMCONV_NAD27				1927
#define	OLD_DATUMCONV_NAD83				1983

#define	OLD_TPARM_FalseEasting	6
#define	OLD_TPARM_FalseNorthing	7

//! Legacy projection parameter structure for use when reading/creating legacy files.
struct OLDPROJPARM {
	INT16 projection;
	MIUNICODE oldprojname[35];
	INT16 zone;
	INT16 system;
	INT16 datumconv;
	INT16 altproj;
	double tparm[15];
	OLDPROJPARM (
		) { memset(this,0,sizeof(*this)); projection = OLD_PROJECTION_Local; }
	};

#endif	// INC_MI32_OLDPROJPARM_H
