/**
 * \file mi32/rasterfilterparms.h
 *
 * \if NODOC
 * $Id: rasterfilterparms.h_v 1.3 2008/04/09 16:13:44 scowan Exp $
 *
 * $Log: rasterfilterparms.h_v $
 * Revision 1.3  2008/04/09 16:13:44  scowan
 * Fixed unix warnings.
 *
 * Revision 1.2  2007/12/27 18:39:44  bcolombe
 * Added CreateFilter methods
 *
 * Revision 1.1  2007/11/19 22:03:37  bcolombe
 * Initial revision
 *
 *
 * \endif
****************************************************************************/

#ifndef	INC_MI32_RASTERFILTERPARMS_H
#define	INC_MI32_RASTERFILTERPARMS_H

#ifndef INC_RVC_SPATFILT_H
#include <rvc/spatfilt.h>
#endif

#ifndef INC_SPATMOD_IMAGESTAGEAUTOPTR_H
#include <spatmod/imagestageautoptr.h>
#endif

#ifndef INC_SPATMOD_IMAGESTAG_H
#include <spatmod/imagestage.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class SERIALIZERITEM;
class MIMATRIX;
#endif

//! Class to hold settings for raster spatial filter
class RASTER_FILTER_PARMS {
	public:
		
		//! Gradient Types
		enum GRADIENTTYPE {
			GRADIENTTYPE_Sobel =	0,
			GRADIENTTYPE_Roberts =	1,
			GRADIENTTYPE_Prewitt =	2,
			GRADIENTTYPE_UserDefined =	3
			};

		//! Default Constructor
		RASTER_FILTER_PARMS (
			);

		//! Destructor
		~RASTER_FILTER_PARMS (
			);

		//! Return amount of filter to be applied
		double GetAmount() const {return m_Amount;}

		//! Set amount of filter to be applied
		void SetAmount(double Amount) {m_Amount = Amount;}

		//! Get kernel A
		void GetMatrixA(MIMATRIX& Matrix) const;

		//! Set kernel A
		void SetMatrixA(const MIMATRIX& MatrixA);

		//! Get kernel B
		void GetMatrixB(MIMATRIX& Matrix) const;

		//! Set kernel B
		void SetMatrixB(const MIMATRIX& MatrixB);

		//! Get row size of kernel
		UINT32 GetNumRows() const {return m_NumRows;}

		//! Set row size of kernel
		void SetNumRows(UINT32 NumRows) {m_NumRows = NumRows;}

		//! Get column size of kernel
		UINT32 GetNumColumns() const {return m_NumColumns;}

		//! Set column size of kernel
		void SetNumColumns(UINT32 NumColumns) {m_NumColumns = NumColumns;}

		//! Get filter type
		RVC::SPATIALFILTER::FILTERTYPE GetFilterType() const {return m_FilterType;}

		//! Set filter type
		void SetFilterType(RVC::SPATIALFILTER::FILTERTYPE FilterType) {m_FilterType = FilterType;}

		//! Get gradient filter type
		GRADIENTTYPE GetGradientType() const {return m_GradientType;}

		//! Set gradient filter type
		void SetGradientType(GRADIENTTYPE GradientType) {m_GradientType = GradientType;}

		//! Get value of slider A
		double GetParamA() const {return m_ParamA;}

		//! Set value of slider A
		void SetParamA(double ParamA) {m_ParamA = ParamA;}

		//! Get value of slider B
		double GetParamB() const {return m_ParamB;}

		//! Set value of slider B
		void SetParamB(double ParamB) {m_ParamB = ParamB;}

		//! Get whether filter is set to be applied
		bool IsEnabled() const {return m_IsEnabled;}

		//! Set wheter filter is to be applied
		void SetEnabled (
			bool bIsEnabled
			) { m_IsEnabled = bIsEnabled; }

		//! Get item definition array for serialization.
		static const SERIALIZERITEM* SerialGetItemDef ();

		//! Return filter_focal stage based on source stage and current parms settings
		ERRVALUE CreateFilter(
			SPATMOD::IMAGE::STAGE *pSourceImage, 
			bool TakeOwnershipSource, 
			SPATMOD::IMAGE::STAGE_AUTOPTR &PtrStage
			);

		//! Return filter_focal stage based on source stage, mask image and current parms settings
		ERRVALUE CreateFilter(
			SPATMOD::IMAGE::STAGE *pSourceImage, 
			bool TakeOwnershipSource, 
			SPATMOD::IMAGE::STAGE *pMaskImage, 
			bool TakeOwnershipMask, 
			SPATMOD::IMAGE::STAGE_AUTOPTR &PtrStage
			);

	private:	
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef RASTER_FILTER_PARMS THISCLASS;

		RVC::SPATIALFILTER::FILTERTYPE m_FilterType; //filter type

		DOUBLE_ARRAY<double> m_ArrayA; //store kernel A as DOUBLE_ARRAY for serialization
		DOUBLE_ARRAY<double> m_ArrayB; //store kernel B as DOUBLE_ARRAY for serialization

		UINT32 m_NumRows; //Number of rows in kernel
		UINT32 m_NumColumns; //Number of columns in kernel
		GRADIENTTYPE m_GradientType; //gradient type
		double m_ParamA; //Value of parameter A
		double m_ParamB; //Value of parameter B
		double m_Amount; //Amount of filter to be applied

		bool m_IsEnabled; //Sets whether filtering is on

		#endif
	};

#endif //INC_MGUI_FORMFILTRAST_H

