/**
 * \file mi32/vdiagram.h
 * \brief Definitions for VORONOI_DIAGRAM class.
 *
 * \if NODOC
 * $Id: vdiagram.h_v 1.3 2011/11/08 13:57:31 mju Exp $
 *
 * $Log: vdiagram.h_v $
 * Revision 1.3  2011/11/08 13:57:31  mju
 * Remove redundand hdr.
 *
 *
 * Revision 1.1  2008/03/10 17:00:15  vdronov
 * Initial revision
 * \endif
**/

#ifndef  INC_VDIAGRAM_H
#define  INC_VDIAGRAM_H

#ifndef  INC_POLYLINE_H
#include <mi32/polyline.h>
#endif

#ifndef  INC_MI32_QTREE_H
#include <mi32/qtree.h>
#endif

#ifndef  INC_MI32_BITSET_H
#include <mi32/bitset.h>
#endif


struct VORONOI_NODE {
	DPOINT2D m_Point;
	SIMPLE_ARRAY<INT32>  m_Edges;					//!< Sorted clockwise
	VORONOI_NODE () {}
	};

struct VORONOI_EDGE {
	INT32 m_BeginNode;								//!< begin node			
	INT32 m_EndNode;									//!< end node				
	INT32 m_LeftPolygon;								//!< left poly			
	INT32 m_RightPolygon;							//!< right poly	
	VORONOI_EDGE () : m_BeginNode(-1), m_EndNode(-1), m_LeftPolygon(-1), m_RightPolygon(-1) {}
	};

struct VORONOI_POLYGON {
	SIMPLE_ARRAY<INT32> m_Nodes;					//!< Sorted clockwise
	VORONOI_POLYGON () {}
	};

class VORONOI_DIAGRAM {
	public:

		VORONOI_DIAGRAM ();

		~VORONOI_DIAGRAM ();

		ERRVALUE Compute (
			const POLYLINE& polyline,
			const DRECT2D& extents
			);

		ERRVALUE Compute (
			const POLYLINE& polyline,
			const double factor
			);

		ERRVALUE ComputePolylineListFromSelectedPolygons (
			const BITSET& bitset,
			POLYLINELIST& polylinelist
			);

		INT32 GetNumNodes (
			) const { return m_Nodes.GetNumItems(); }

		INT32 GetNumEdges (
			) const { return m_Edges.GetNumItems(); }

		INT32 GetNumPolygons (
			) const { return m_Polygons.GetNumItems(); }

		const VORONOI_NODE& GetNode (
			const INT32 index
			) const { return (*(m_Nodes[index])); }

		const VORONOI_EDGE& GetEdge (
			const INT32 index
			) const { return (m_Edges[index]); }

		const VORONOI_POLYGON& GetPolygon (
			const INT32 index
			) const { return (*(m_Polygons[index])); }

		bool IsComputed (
			) const { return m_IsComputed; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		SIMPLE_ARRAY<VORONOI_NODE*> m_Nodes;
		SIMPLE_ARRAY<VORONOI_EDGE> m_Edges;
		SIMPLE_ARRAY<VORONOI_POLYGON*> m_Polygons;
		bool m_IsComputed;

		void Clear ();

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#endif  // INC_VDIAGRAM_H
