/**
 * \file rvc/dbnode.h 
 * \brief interface for the RVC::DBASE_NODE class.
 *
 * \if NODOC
 * $Id: dbnode.h_v 1.13 2010/05/12 21:07:35 mju Exp $
 *
 * $Log: dbnode.h_v $
 * Revision 1.13  2010/05/12 21:07:35  mju
 * Simplify and specify parent objtype when construct base.
 *
 * Revision 1.10  2006/06/02 22:23:35  dwilliss
 * Changes to Join
 *
 * Revision 1.5  2003/01/17 20:13:45  scowan
 * Added copy method.
 *
 * Revision 1.1  2001/05/24 15:57:36  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef INC_RVC_DBNODE_H
#define INC_RVC_DBNODE_H

#ifndef INC_RVC_DBELEM_H
	#include <rvc/dbelem.h>
#endif

namespace RVC {

#ifndef GENERATING_DOXYGEN_OUTPUT
class VECTOR;			// Forward declarations
class GEOMETRIC;		// Forward declarations
#endif // GENERATING_DOXYGEN_OUTPUT

//! Manage database associated with vector node elements.
class DBASE_NODE : public RVC::DBASE_ELEMENT {
	public:

		//! Default constructor
		DBASE_NODE (
			): DBASE_ELEMENT(OBJTYPE_Vector,ELEMTYPE_Node) { }

		//! Copy constructor
		DBASE_NODE (
			const RVC::DBASE_NODE& rhs
			): DBASE_ELEMENT(rhs) { }

		//! Destructor
		virtual ~DBASE_NODE (
			);

		//! Assignment
		RVC::DBASE_NODE& operator= (
			const RVC::DBASE_NODE& rhs
			);
			
		//! Copy a database from the source to under the destination object using an element translation list
		ERRVALUE CopyDatabaseTo (
			RVC::GEOMETRIC& DestParentObj,
			const SIMPLE_ARRAY<INT32>& PointTransList,
			const SIMPLE_ARRAY<INT32>& NodeTransList,
			COPYFLAG flags
			);

		//! Determine that the node has attributes and will be converted 
		//! to a point if all of the lines are removed from the node
		//! @return 'True' if the node is considered a point, false if not
		bool IsNodeAPoint (
			INT32 NodeNum
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		DBASE_ELEMENT::DBLIST m_DBList;		// Used for IsNodeAPoint()

		#endif // GENERATING_DOXYGEN_OUTPUT

	};

}		//! End of RVC namespace

#endif 	//!< INC_RVC_DBNODE_H
