/**
 * \file rvc/freqproc.h 
 * \brief interface for the RVC::FREQPROC class.
 *
 * \if NODOC
 * $Id: freqproc.h_v 1.12 2007/09/06 17:18:46 scowan Exp $
 *
 * $Log: freqproc.h_v $
 * Revision 1.12  2007/09/06 17:18:46  scowan
 * Read and write virtual methods changed to take objopenparms.
 *
 * Revision 1.11  2005/12/09 17:29:27  scowan
 * Updated to use credentials.
 *
 * Revision 1.10  2005/07/19 17:49:01  scowan
 * Update to use current simple methods.
 *
 * Revision 1.9  2005/01/19 23:39:00  scowan
 * Support of descriptor class.
 *
 * Revision 1.8  2003/10/07 17:32:50  scowan
 * Object virtual method changes.
 *
 * Revision 1.7  2003/09/30 16:52:06  dwilliss
 * Doxygen
 *
 * Revision 1.6  2003/09/25 21:20:04  dwilliss
 * Fixed doxygen comment
 *
 * Revision 1.5  2003/09/15 13:49:45  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.4  2001/06/15 18:58:41  scowan
 * More const.
 *
 * Revision 1.3  2001/06/14 21:35:29  scowan
 * Added to RVC namespace.
 *
 * Revision 1.2  2000/10/19 17:52:43  scowan
 * Checked in for vacation.
 *
 * Revision 1.1  2000/09/29 19:31:33  scowan
 * Initial revision
 *
 * \endif
**/

//!	\class RVC::FREQPROC rvc/freqproc.h
//!	\brief Manages RVC::OBJTYPE_FreqProcInfo objects.  
//!
//!	The interface currently wraps the MfFreqProc functions.
//!
//!	This class inherits from RVC::SIMPLE
//!	
//!	RVC Object Types that this class supports: RVC::OBJTYPE_FreqProcInfo
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_FreqProcInfo
//!
//!	Valid parents for this class:
//!		- OBJTYPE_Raster
//!
//!	Read() and Write() methods inherited from RVC::SIMPLE class.  View RVC:::SIMPLE docs for inherited methods

#ifndef INC_RVC_FREQPROC_H
#define INC_RVC_FREQPROC_H

#ifndef INC_RVC_SIMPLE_H
	#include <rvc/simple.h>
#endif

#ifndef INC_MI32_RECT_H
	#include <mi32/rect.h>
#endif

namespace RVC {

class FREQPROC : public RVC::SIMPLE {
	public:
		FREQPROC (
			);

		FREQPROC (									//! Copy Constructor
			const RVC::FREQPROC& rhs					
			);										
													
		virtual ~FREQPROC (
			);

		RVC::FREQPROC& operator= (						//! Assignment operator
			const RVC::FREQPROC& rhs
			);

		//! Obtain boundary rect
		//! @ return Boundary rectangle
		const LRECT2D& GetBoundRect (
			) const;

		//! Set boundary rect
		void SetBoundRect (
			const LRECT2D& BoundRect
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;
		
		//! Implementation of RVC::SIMPLE virtual methods
		virtual ERRVALUE v_GetDftNameDesc (const OBJECT& parent, DESCRIPTOR& Descriptor);
		virtual ERRVALUE v_MakeData (RVC::OBJMAKEPARMS& ObjMakeParms);
		virtual ERRVALUE v_ReadData (RVC::OBJOPENPARMS& ObjOpenParms);
		virtual ERRVALUE v_WriteData (RVC::OBJOPENPARMS& ObjOpenParms);

		LRECT2D m_BoundRect;
		INT8 m_Mode;
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

}		//! End of RVC namespace

#endif		//!< INC_RVC_FREQPROC_H
