/**
 * \file rvc/statdbfield.h 
 * \brief interface for the RVC::STAT_DBFIELD class.
 *
 * \if NODOC
 * $Id: statdbfield.h_v 1.1 2010/05/20 21:43:28 mju Exp $
 *
 * $Log: statdbfield.h_v $
 * Revision 1.1  2010/05/20 21:43:28  mju
 * Initial revision
 *
 * \endif
**/

#ifndef INC_RVC_STATDBFIELD_H
#define INC_RVC_STATDBFIELD_H

#ifndef INC_RVC_OBJECT_H
	#include <rvc/object.h>
#endif

#ifndef INC_MI32_STATUNIVAR_H
	#include <mi32/statunivar.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward Declarations
namespace RVC {
	class CREDENTIALS;
	class FILEHANDLE;
	class DBTABLE;
	struct OBJHDR_STATDBFIELD;
	}
#endif		// GENERATING_DOXYGEN_OUTPUT

namespace RVC {

//! Manages RVC::OBJTYPE_Stat_DbField objects.
//!
//! Valid parents for this class: RVC::OBJTYPE_Table
class STAT_DBFIELD : public RVC::OBJECT {
	public:

		class FILTER_FIELDNUM;

		//! Find and read statistics for field if exist.
		//! @return TRUE if exists, FALSE if not, < 0 if error.
		static int FindAndRead (
			const RVC::DBTABLE& ObjTable,			//!< Table
			INT32 FieldNum,							//!< Field number
			STAT_UNIVARIATE& Stats					//!< Statistics returned
			);

		//! Default constructor.	
		STAT_DBFIELD (
			);

		//! Copy constructor.
		STAT_DBFIELD (
			const STAT_DBFIELD& rhs
			);

		//! Destructor.
		virtual ~STAT_DBFIELD (
			);

		//! Assignment.
		STAT_DBFIELD& operator= (
			const STAT_DBFIELD& rhs
			);

		//! Get field number.
		INT32 GetFieldNum () const
			{ return (m_FieldNum); }

		//! Get statistics.
		const STAT_UNIVARIATE& GetStatistics () const
			{ return (m_Stats); }

		//! Read statistics for field if exists.
		ERRVALUE Read (
			const RVC::DBTABLE& ObjTable,
			INT32 FieldNum,
			const CREDENTIALS *pCredentials RVC_DEFAULT_CREDENTIALS	//!< User credentials passed in, NULL for default behavior
			);

		//! Write object with stats for specified field.
		ERRVALUE Write (
			const RVC::DBTABLE& ObjTable,
			INT32 FieldNum,
			const STAT_UNIVARIATE& Stats,
			const CREDENTIALS *pCredentials RVC_DEFAULT_CREDENTIALS	//!< User credentials passed in, NULL for default behavior
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		INT32 m_FieldNum;
		STAT_UNIVARIATE m_Stats;

		static ERRVALUE ReadHeader (FILEHANDLE *pFileHandle, const OBJECT& obj, OBJHDR_STATDBFIELD& objhdr);

		// Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//! Filter to determine which object to select based on the field number.
class STAT_DBFIELD::FILTER_FIELDNUM : public OBJECT::FILTER {
	public:

		//! Constructor with parent table and field number.
		FILTER_FIELDNUM (
			const DBTABLE& ObjTable,
			INT32 FieldNum
			);

		//! Destructor
		virtual ~FILTER_FIELDNUM ();
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FILEHANDLE *m_pFileHandle;
		INT32 m_FieldNum;
		
		virtual bool Filter (const RVC::OBJECT& obj);
		#endif		// GENERATING_DOXYGEN_OUTPUT
	};

}		// End namespace RVC

#endif		// INC_RVC_STATDBFIELD_H
