/**
 * \file rvc/uiformimgdisplaybands.h
 * \brief RVC::UI::FORM_IMAGEDISPLAYBANDS definition.
 *
 * \if NODOC
 * $Id: uiformimgdisplaybands.h_v 1.2 2010/10/11 12:50:21 mju Exp $
 *
 * $Log: uiformimgdisplaybands.h_v $
 * Revision 1.2  2010/10/11 12:50:21  mju
 * Change design so add items before create.
 *
 * Revision 1.1  2010/10/08 13:07:31  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_UIFORMIMGDISPLAYBANDS_H
#define	INC_RVC_UIFORMIMGDISPLAYBANDS_H

#ifndef	INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

#ifndef	INC_RVC_OBJITEMLIST_H
#include <rvc/objitemlist.h>
#endif

namespace RVC {
namespace UI {

//! Form for choosing image display mode and bands.
class FORM_IMAGEDISPLAYBANDS : public MGUI::FORM_COMPOSITE {
	public:

		enum MODE {
			MODE_Auto,
			MODE_Single,
			MODE_RGB
			};

		//! Constructor.
		FORM_IMAGEDISPLAYBANDS ();

		//! Destructor.
		~FORM_IMAGEDISPLAYBANDS ();

		//! Add item for selection.
		//! Must be called before Create().
		void AddItem (
			const RVC::OBJITEM& ObjItem
			) { m_ObjItemListPickFrom.PushBack(ObjItem); }

		//! Create form.
		//! Must use AddItem and, optionally, SetSingle/RGB before calling.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane
			);

		//! Get current mode.
		MODE GetMode () const
			{ return (m_Mode); }

		//! Get list of selected items valid for current mode.
		const RVC::OBJITEMLIST& GetObjItemListSelected () const
			{ return (m_ObjItemListSelected); }

		//! Set delegate to call whenever user makes a change.
		void SetDelegateOnChange (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnChange = delegate; }

		//! Set to RGB objects.
		void SetRGB (
			const RVC::OBJITEM& ObjItemRed,
			const RVC::OBJITEM& ObjItemGreen,
			const RVC::OBJITEM& ObjItemBlue
			);

		//! Set to single object.
		void SetSingle (
			const RVC::OBJITEM& ObjItem
			);

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		typedef FORM_IMAGEDISPLAYBANDS THISCLASS;

		RVC::OBJITEMLIST m_ObjItemListPickFrom;
		RVC::OBJITEMLIST m_ObjItemListSelected;
		MODE m_Mode;
		MGUI::CTRL_TOGGLEBUTTON m_TglSingle;
		MGUI::CTRL_TOGGLEBUTTON m_TglRGB;
		MGUI::CTRL_COMBOBOX m_CbxSingle;
		MGUI::FORM_COMBOBOX m_CbxRed;
		MGUI::FORM_COMBOBOX m_CbxGreen;
		MGUI::FORM_COMBOBOX m_CbxBlue;
		DELEGATE_VOID_NOPARMS m_DelegateOnChange;

		void NotifyChange () const
			{ if (m_DelegateOnChange) m_DelegateOnChange(); }
		void OnCbxBlue ();
		void OnCbxGreen ();
		void OnCbxRed ();
		void OnCbxSingle ();
		void OnTglRGB ();
		void OnTglSingle ();
		#endif
	};

}	// End namespace UI
}	// End namespace RVC

#endif	// INC_RVC_UIFORMIMGDISPLAYBANDS_H
