/******************************************************************************
 *
 * \file sml/callbackstruct.h
 * \brief SML CALLBACKSTRUCT
 *
 * \if NODOC
 * $Log: callbackstruct.h_v $
 * Revision 1.2  2007/05/08 15:45:22  dwilliss
 * SML namespace
 *
 * Revision 1.1  2007/05/04 22:03:13  dwilliss
 * Initial revision
 *
 * \endif
*/
/*****************************************************************************/

#ifndef INC_SML_CALLBACKSTRUCT_H
#define INC_SML_CALLBACKSTRUCT_H

#ifndef	INC_MI32_MICALLBACKLIST_H
	#include <mi32/micallbacklist.h>
#endif
#ifndef	INC_RVC_OBJITEM_H
	#include <rvc/objitem.h>
#endif
#ifndef	INC_MI32_ELEMTYPE_H
#include <mi32/elemtype.h>
#endif

namespace SML {
	class CONTEXT;
	class SYMBOL;

/***************************************************************************/

//! Structure passed to an SML callback
//!
//! <b>Possible reasons:</b>
//! \li	<b>MsmlCR_Modified</b> Script has been modified
//! \li <b>MsmlCR_Filename</b> Filename has changed
//! \li <b>MsmlCR_SymbolAdded</b> A symbol has been added
//! \li <b>MsmlCR_End</b> The script has finished running
//! \li <b>MsmlCR_DestroyBegin</b> The context is about to be destoied
//!
//! The following reasons can only be added by calling the static method
//! SML::CONTEXT::CallbackAddGlobal() and will be called for every context
//! which is created and destroied.
//!
//! \li <b>MsmlCR_Create</b> A context is being created
//! \li <b>MsmlCR_Destroy</b> A context has been destroied (The context member of the SML::CALLBACKSTRUCT will be NULL, since it's already gone)
//! \li <b>MsmlCR_InstallFuncsSML</b> Install any functions which should only
//! 	be used in an SML context (as opposed to a query context)
//!
//! @note The reasons are flags which may be OR'd together.  It is
//! possible to get one callback with multiple reasons set
//!
//! @see SML::CONTEXT::CallbackAdd()
//! @see SML::CONTEXT::CallbackRemove()
//! @see SML::CONTEXT::CallbackAddGlobal()
//! @see SML::CONTEXT::CallbackRemoveGlobal()

struct CALLBACKSTRUCT : public MICALLBACKLIST::ANYCALLBACKSTRUCT {
	CALLBACKSTRUCT (
		) :
		ChangesMade(0),
		context(0),
		sym(0),
		err(0)
		{
		}

	CALLBACKSTRUCT (
		CONTEXT *pcontext,
		UINT32 preason
		) :
		ANYCALLBACKSTRUCT(preason),
		ChangesMade(0),
		context(pcontext),
		sym(0),
		err(0)
		{
		}
#ifndef GENERATING_DOXYGEN_OUTPUT
	// These used to be filename/inode which is now ObjItem at the bottom.
	// But because we construct these inside RVCSYS and pass them out, we 
	// can't change the offset of the other members without recompiling 
	// everything.  So delete these late some friday afternoon just before
	// a weekend build. And while were're at it, change ChangedMade to a bool.
	// I'd do that now, but sizeof(bool) may not be == sizeof(int) on all
	// systems
	MIUNICODE* xxUnused1;
	INT32 xxUnsued2;
#endif
	int ChangesMade;			//!< true if script modified since last save /load
	SML::CONTEXT *context;		//!< Context of the SML script/query
	SML::SYMBOL *sym;			//!< For MsmlCR_SymbolAdded
	ERRVALUE err;
	RVC::OBJITEM ObjItem;	//! ObjItem of script (MsmlCR_Modified only) Will return true from IsFileOnly() if it's not an RVC object
	};

PREVENT_MEMFUNC(SML::CALLBACKSTRUCT)


} // end of SML namespace

typedef SML::CALLBACKSTRUCT SMLCALLBACKSTRUCT;


#define MsmlCR_Modified				0x00000001
#define MsmlCR_Filename				0x00000002
#define MsmlCR_SymbolAdded			0x00000004
#define MsmlCR_End					0x00000008
#define MsmlCR_DestroyBegin		0x00000010
#define MsmlCR_Create				0x00000020	//!< Any context being created
#define MsmlCR_Destroy				0x00000040	//!< Any context being destroied
#define MsmlCR_InstallFuncsSML	0x00000080	//!< An SML-only context being created
#define MsmlCR_InstallFuncsGRE	0x00000100	//!< A GRE-only context being created

#endif		//  INC_SML_CALLBACKSTRUCT_H


